#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
@author : Mohan
email : mohan88@ufl.edu

This file tests for the function verify_and_correct_collection_date

'''
import unittest
import os
import sys
file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'
sys.path.append(proj_root + 'redi/')
from lxml import etree
from redi import redi

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestVerifyAndCorrectCollectionDate(unittest.TestCase):

    def setUp(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        
    def test_verify_and_correct_collection_date_case1(self):
        self.sampleData = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <DATE_TIME_STAMP>7891-11-30 11:12:00</DATE_TIME_STAMP>
            <RESULT_DATE>7891-11-30 11:12:00</RESULT_DATE>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <DATE_TIME_STAMP></DATE_TIME_STAMP>
            <RESULT_DATE>1909-08-27 16:13:00</RESULT_DATE>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        self.sampleData_tree= etree.ElementTree(etree.fromstring(self.sampleData))

        self.result = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <DATE_TIME_STAMP>7891-11-30 11:12:00</DATE_TIME_STAMP>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <DATE_TIME_STAMP>1909-08-23 16:13:00</DATE_TIME_STAMP>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        input_date_format = '%Y-%m-%d %H:%M:%S'
        result_from_method = redi.verify_and_correct_collection_date(self.sampleData_tree, input_date_format)
        result_str = etree.tostring(result_from_method[0], method='xml', pretty_print=True)
        self.assertEqual(self.result, result_str)

    def test_verify_and_correct_collection_date_case2(self):
        self.sampleData = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <RESULT_DATE>7891-11-30 11:12:00</RESULT_DATE>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        self.sampleData_tree= etree.ElementTree(etree.fromstring(self.sampleData))

        self.result = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <DATE_TIME_STAMP>7891-11-26 11:12:00</DATE_TIME_STAMP><STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        input_date_format = '%Y-%m-%d %H:%M:%S'
        result_from_method = redi.verify_and_correct_collection_date(self.sampleData_tree, input_date_format)
        result_str = etree.tostring(result_from_method[0], method='xml', pretty_print=True)
        self.assertEqual(self.result, result_str)

    def test_verify_and_correct_collection_date_case3(self):
        # for date_time_stamp falling in february of a non leap year
        self.sampleData = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <DATE_TIME_STAMP>7891-11-30 11:12:00</DATE_TIME_STAMP>
            <RESULT_DATE>7891-11-30 11:12:00</RESULT_DATE>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <DATE_TIME_STAMP></DATE_TIME_STAMP>
            <RESULT_DATE>1909-03-01 16:13:00</RESULT_DATE>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        self.sampleData_tree= etree.ElementTree(etree.fromstring(self.sampleData))

        self.result = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <DATE_TIME_STAMP>7891-11-30 11:12:00</DATE_TIME_STAMP>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <DATE_TIME_STAMP>1909-02-25 16:13:00</DATE_TIME_STAMP>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        input_date_format = '%Y-%m-%d %H:%M:%S'
        result_from_method = redi.verify_and_correct_collection_date(self.sampleData_tree, input_date_format)
        result_str = etree.tostring(result_from_method[0], method='xml', pretty_print=True)
        self.assertEqual(self.result, result_str)
        
    def test_verify_and_correct_collection_date_case4(self):
        # for date_time_stamp falling in february of a leap year
        self.sampleData = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <RESULT_DATE>8000-03-01 11:12:00</RESULT_DATE>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        self.sampleData_tree= etree.ElementTree(etree.fromstring(self.sampleData))

        self.result = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <DATE_TIME_STAMP>8000-02-26 11:12:00</DATE_TIME_STAMP><STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        input_date_format = '%Y-%m-%d %H:%M:%S'
        result_from_method = redi.verify_and_correct_collection_date(self.sampleData_tree, input_date_format)
        result_str = etree.tostring(result_from_method[0], method='xml', pretty_print=True)
        self.assertEqual(self.result, result_str)

    def test_verify_and_correct_collection_date_case5(self):
        self.sampleData = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
    </study>
"""
        self.sampleData_tree= etree.ElementTree(etree.fromstring(self.sampleData))

        self.result = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
    </study>
"""
        input_date_format = '%Y-%m-%d %H:%M:%S'
        result_from_method = redi.verify_and_correct_collection_date(self.sampleData_tree, input_date_format)
        result_str = etree.tostring(result_from_method[0], method='xml', pretty_print=True)
        self.assertEqual(self.result, result_str)

    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

"""
Functions related to the RediBatch database
"""

import datetime
import hashlib
import logging
import os
import sqlite3 as lite
import stat
import sys
import time

from lxml import etree

from utils import redi_email
from utils.rawxml import RawXml


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())
BATCH_STATUS_STARTED = 'Started'
BATCH_STATUS_COMPLETED = 'Completed'

"""
@see #check_input_file()

The first time we run the app there is no SQLite file
where to store the md5 sums of the input file.
This function creates an empty RediBatch in the SQLite
file specified as `db_path`

@return True if the database file was properly created with an empty table
"""

def create_empty_md5_database(db_path) :
    if os.path.exists(db_path) :
        logger.warn('The file with name ' + db_path + ' already exists')
        #return

    try :
        logger.info('Opening the file:' + db_path)
        fresh_file = open(db_path, 'w')
        fresh_file.close()
        os.chmod(db_path, stat.S_IRUSR | stat.S_IWUSR)
        time.sleep(5)

    except IOError as e:
        logger.error("I/O error: " + e.strerror + ' for file: ' + db_path)
        return False
    success = create_empty_table(db_path)
    return success

"""
Helper for #create_empty_md5_database()
"""

def create_empty_table(db_path) :
    logger.info('exec create_empty_table')
    db = None
    try:
        db = lite.connect(db_path)
        cur = db.cursor()
        sql = """CREATE TABLE RediBatch (
    rbID INTEGER PRIMARY KEY AUTOINCREMENT,
    rbCreateTime DATETIME DEFAULT CURRENT_TIMESTAMP,
    rbStartTime DATETIME,
    rbEndTime DATETIME,
    rbStatus TEXT,
    rbMd5Sum TEXT NOT NULL
)
        """
        cur.execute(sql)

    except lite.Error as e:
        logger.error("SQLite error in create_empty_table(): " + e.args[0])
        return False
    finally:
        if db:
            db.close()
    logger.info('success create_empty_table')
    return True


"""
Use this function to set the `row_factory`
attribute of the database connection
"""


def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d

"""
@see bin/redi.py#main()
@return a dictionary representation of the batch row for the current run

Check the md5sum of the input file
    - if the sum *has changed* then continue the data processing and store a row
        in the SQLite database with `batch status= started/ completed`

    - if the sum *did not change* then check the config option `batch_warning_days`:
        - if       limit = -1       then continue execution (ignore the limit)
        - if days_passed > limit    then stop the process and email the `redi_admin`

"""


def check_input_file(batch_warning_days, db_path, email_settings, raw_xml_file, project, start_time):
    batch = None

    if not os.path.exists(db_path) :
        create_empty_md5_database(db_path)

    new_md5ive = get_md5_input_file(raw_xml_file)
    new_msg = 'Using SQLite file: %s to store input file: %s md5 sum: %s' % (
        db_path, raw_xml_file, new_md5ive)
    logger.info(new_msg)

    old_batch = get_last_batch(db_path)
    old_md5ive = None
    if old_batch:
        old_md5ive = old_batch['rbMd5Sum']
        logger.info('Old md5 sum for the input file is: ' + old_md5ive)
    else:
        # this is the first time the checksum feature is used
        logger.info(
            "There is no old md5 recorded yet for the input file. Continue data import...")
        batch = add_batch_entry(db_path, new_md5ive)
        record_msg = 'Added batch (rbID= %s, rbCreateTime= %s, rbMd5Sum= %s' % (
            batch['rbID'], batch['rbCreateTime'], batch['rbMd5Sum'])
        logger.info(record_msg)
        return batch

    if old_md5ive != new_md5ive:
        # the data has changed... insert a new batch entry
        batch = add_batch_entry(db_path, new_md5ive)
        record_msg = 'Added batch (rbID= %s, rbCreateTime= %s, rbMd5Sum= %s' % (
            batch['rbID'], batch['rbCreateTime'], batch['rbMd5Sum'])
        logger.info(record_msg)
        return batch
    else:
        days_since_today = get_days_since_today(old_batch['rbCreateTime'])
        # TODO: refactor code to use ConfigParser.RawConfigParser in order to
        # preserve data types

        if (days_since_today > int(batch_warning_days)):
            raw_xml = RawXml(project, raw_xml_file)
            msg_file_details = "\nXML file details: " + raw_xml.get_info()
            logger.info('Last import was started on: %s which is more than '\
                ' the limit of %s' % (old_batch['rbStartTime'], batch_warning_days))
            if (-1 == int(batch_warning_days)):
                msg_continue = """
                The configuration `batch_warning_days = -1` indicates that we want to continue
                execution even if the input file did not change
                """ + msg_file_details
                logger.info(msg_continue)
            else:

                msg_quit = "The input file did not change in the past: %s days." % days_since_today
                logger.critical(msg_quit + msg_file_details)
                redi_email.send_email_input_data_unchanged(email_settings, raw_xml)
                sys.exit()
        else:
            logger.info('Reusing md5 entry: ' + str(old_batch['rbID']))
    # return the old batch so we can update the status
    return old_batch



"""
Retrieve the row corresponding to the last REDI batch completed
"""


def get_last_batch(db_path):
    batch = None
    try:
        db = lite.connect(db_path)
        db.row_factory = dict_factory
        cur = db.cursor()
        sql = """
SELECT
    rbID, rbCreateTime, rbStartTime, rbEndTime, rbMd5Sum
FROM
    RediBatch
ORDER BY rbID DESC
LIMIT 1
"""
        cur.execute(sql)
        batch = cur.fetchone()

    except lite.Error as e:
        logger.error("SQLite error in get_last_batch() for file %s - %s" % (db_path, e.args[0]))
        return None
    finally:
        if db:
            db.close()

    return batch


def get_batch_by_id(db_path, batch_id):
    db = None
    """
    Retrieve the row corresponding to the specified primary key
    """
    try:
        db = lite.connect(db_path)
        db.row_factory = dict_factory
        cur = db.cursor()
        sql = """
SELECT
    rbID, rbCreateTime, rbStartTime, rbEndTime, rbMd5Sum
FROM
    RediBatch
WHERE
    rbID = ?
LIMIT 1
"""
        cur.execute(sql, (str(batch_id), ))
        batch = cur.fetchone()

    except lite.Error as e:
        logger.exception("SQLite error in get_batch_by_id(): %s:" % e.args[0])
        raise
        # sys.exit(1)
    finally:
        if db:
            db.close()

    return batch



def get_md5_input_file(input_file):
    """
    @see #check_input_file()
    @see https://docs.python.org/2/library/hashlib.html
    @see https://docs.python.org/2/library/sqlite3.html#sqlite3.Connection.row_factory

    Returns the md5 sum for the redi input file
    """
    if not os.path.exists(input_file):
        raise Exception('Input file not found at: ' + input_file)

    logger.info('Computing md5 sum for: ' + input_file)

    # open the file in binary mode
    f = open(input_file, 'rb')
    chunk_size = 2 ** 20
    md5 = hashlib.md5()

    # read the input file in 1MB pieces
    while True:
        chunk = f.read(chunk_size)
        if not chunk:
            break
        md5.update(chunk)

    return md5.hexdigest()


def add_batch_entry(db_path, md5):
    """
    Inserts a row into RediBatch table
    @see #check_input_file()
    Parameters
    ----------
    db_path : string
        The SQLite database file name
    md5 : string
        The md5 sum to be inserted
    create_time : string
        The batch start time
    """
    try:
        db = lite.connect(db_path)
        db.row_factory = dict_factory
        cur = db.cursor()
        sql = """
INSERT INTO RediBatch
    (rbCreateTime,rbStartTime, rbEndTime, rbStatus, rbMd5Sum)
VALUES
    ( ?, NULL, NULL, ?, ?)
"""
        create_time = get_db_friendly_date_time()
        cur.execute(sql, (create_time, BATCH_STATUS_STARTED, md5))
        rbID = cur.lastrowid
        db.commit()
        batch = get_batch_by_id(db_path, rbID)

    except lite.Error as e:
        logger.error("SQLite error in add_batch_entry() for file %s - %s" % (db_path, e.args[0]))
        return None
    finally:
        if db:
            db.close()

    return batch


def update_batch_entry(db_path, id, status, start_time, end_time):
    """
    Update the status and the start/end time of a specified batch entry
    Return True if update succeeded, False otherwise

    Parameters
    ----------
    db_path : string
    id : integer
    status : string
    start_time : datetime string
    end_time : datetime string
    """
    try:
        db = lite.connect(db_path)
        cur = db.cursor()
        sql = """
UPDATE
    RediBatch
SET
    rbStartTime = ?
    , rbEndTime = ?
    , rbStatus = ?
WHERE
    rbID = ?
"""

        cur.execute(sql, (start_time, end_time, status, id))
        db.commit()
        success = True
    except lite.Error as e:
        logger.exception("SQLite error in update_batch_entry(): %s:" % e.args[0])
        success = False
    finally:
        if db:
            db.close()

    return success


def get_db_friendly_date_time():
    """
    @return string in format: "2014-06-24 01:23:24"
    """
    return datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')


def get_db_friendly_date():
    """
    @return string in format: 2014-06-24
    """
    return datetime.date.today()

def get_days_since_today(date_string):
    """
    @return the number of days passed since the specified date
    """
    num = None
    other = datetime.datetime.strptime(date_string, '%Y-%m-%d %H:%M:%S')
    now = datetime.datetime.now()
    delta = now - other
    return delta.days

def printxml(tree):
    """
    Helper function for debugging xml content
    """
    print etree.tostring(tree, pretty_print = True)
    return



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import smtplib
from smtplib import SMTPException
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email import Encoders
from datetime import date
import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

"""
This module groups together related functions for
sending emails.

Module unit test: test/TestRediEmail.py
"""

def send_email_redcap_connection_error(email_settings, subject='', msg=''):
    """
    Return True if the email was sent.
    Notify the designated `REDCap support` person about problems
    with reaching REDCap

    Parameters
    ----------
    email_settings : dict
        The dictionary with smtp server parameters
    subject : str
        The email subject
    msg : str
        The content to be emailed
    """

    sender = email_settings['redcap_support_sender_email']
    to_addr_list = email_settings['redcap_support_receiving_list']
    host = email_settings['smtp_host_for_outbound_mail']
    port = email_settings['smtp_port_for_outbound_mail']
    subject = 'Communication failure: Unable to reach REDCap instance'
    msg = 'A problem was encountered when connecting to the REDCap. Please investigate if REDCap is running.'

    logger.error('Exception: Unable to communicate with REDCap instance at: ' + email_settings['redcap_uri'])
    return send_email(host, str(port), sender, to_addr_list, None, subject, msg)

def send_email_input_data_unchanged(email_settings, raw_xml):
    """
    Send a warning email to the `redcap_support_receiver_email`
    if the input file did not change for more than `batch_warning_days`
    Return True if the email was sent

    Parameters
    ----------
    email_settings : dictionary
        The email delivery parameters
    raw_xml : RawXml instance
        The object storing details about the input file
    """
    sender = email_settings['redcap_support_sender_email']
    to_addr_list = email_settings['redcap_support_receiving_list']
    host = email_settings['smtp_host_for_outbound_mail']
    port = email_settings['smtp_port_for_outbound_mail']
    subject = "The data for '{0}' project did not change in more than {1} days.".format(raw_xml.get_project(), email_settings['batch_warning_days'])
    msg = """
Administrators,
    """ +  subject + """

Please check if the input xml file is in the proper location.
    """ + raw_xml.get_info()
    return send_email(host, str(port), sender, to_addr_list, None, subject, msg)


def add_attachment(msg, body):
    """
    Add the html report as attachment

    Parameters
    ----------
    msg : MIMEMultipart
        The object to which we attach the body content
    body : string
        The html content to be attached
    """
    part = MIMEBase('application', "octet-stream")
    part.set_payload(body)
    Encoders.encode_base64(part)
    file_name = "redi_report_{}.html".format(date.today())
    part.add_header('Content-Disposition', \
            'attachment; filename="{}"'.format(file_name))
    msg.attach(part)


def send_email_data_import_completed(email_settings, body=''):
    """
    Email the html report after redi completed the data transfer
    Returns a dictionary, with one entry for each recipient that was refused

    Parameters
    ----------
    email_settings : dict
        Email params produced by redi.get_email_settings()
    body : string
        The html content produced by transforming the xsl
        generated by redi.create_summary_report()
    """
    sender = email_settings['batch_report_sender_email']
    to_addr_list = email_settings['batch_report_receiving_list']
    host = email_settings['smtp_host_for_outbound_mail']
    port = email_settings['smtp_port_for_outbound_mail']
    site_name = email_settings['site_name']
    subject = 'Data Import Report For {}'.format(site_name)
    msg = MIMEMultipart()
    msg['From'] = sender
    msg['To'] = ",".join(to_addr_list)
    msg['Subject'] = subject
    msg.attach(MIMEText(body, 'html'))
    add_attachment(msg, body)

    refused_list = {}
    smtp_obj = smtplib.SMTP(host, port)
    refused_list = smtp_obj.sendmail(sender, to_addr_list, msg.as_string())
    logger.info("Successfully sent email to: " + str(to_addr_list))
    smtp_obj.quit()
    return refused_list

def send_email(
        host,
        port,
        sender,
        to_addr_list,
        cc_addr_list,
        subject,
        msg_body):
    """
    The email deliverer. Return True if the email was sent

    Parameters
    ----------
    to_addr_list : list
        The recipients of the email
    """
    #print ('host %s, port: %s' % (host, port))
    success = False
    try:
        smtp = smtplib.SMTP(host, port)
        header = 'From: %s\n' % sender
        header += 'To: %s\n' % ','.join(to_addr_list)
        if cc_addr_list:
            header += 'Cc: %s\n' % ','.join(cc_addr_list)
        if subject:
            header += 'Subject: %s\n\n' % subject
        msg = header + msg_body
        smtp.sendmail(sender, to_addr_list, msg)
        success = True
        logger.info(
            'Success: Email with subject [' +
            subject +
            '] was sent to:' +
            str(to_addr_list))
    except SMTPException as smtpe:
        logger.error("Unable to send email with subject [{0}] to {1} due: {2}" \
            .format(subject, str(to_addr_list), str(smtpe)))
        logger.info("Please check if the recipient email is valid")
    except Exception as e:
        logger.error("Unable to send email with subject [{0}] to {1}\n due: {2}" \
            .format(subject, str(to_addr_list), str(e)))
        logger.info("Please check if the smtp server is configured properly")
    return success



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import os.path
import time
import datetime

class RawXml(object):
    """
    This class is used to store details about the input file
    @see redi.batch.check_input_file()
    """

    def __init__(self, project, path):
        """
        Parameters
        ----------
        project : string
            The project name - the owner of the xml file
        path : string
            The xml file path
        """
        self._project = project
        self._path = path


    def get_project(self):
        return self._project

    def get_creation_time(self):
        """ Get the OS creation time """
        #tst = time.ctime(os.path.getctime(self._path))
        tst = os.path.getctime(self._path)
        return datetime.datetime.fromtimestamp(tst)


    def get_last_modified_time(self):
        """ Get the OS modification time """
        tst = os.path.getmtime(self._path)
        return datetime.datetime.fromtimestamp(tst)


    def get_info(self):
        """
        Return a string containing all details available about the xml file
        """
        info = """
Project name: {0}
File path: {1}
File created: {2}
File last modified: {3} """.format(self._project, \
            self._path, \
            self.get_creation_time(), \
            self.get_last_modified_time())
        return info



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestUpdateFormImported(unittest.TestCase):

    def setUp(self):
        self.sortedData = """
    <study>
    <subject>
        <Study_Id>001-0001</Study_Id>
        <Study_Start>05/08/21</Study_Start>
        <Collection_Date>09/19/17</Collection_Date>
        <Collection_Time>11:57</Collection_Time>
        <Qualifying_Result>Y</Qualifying_Result>
        <Study_Stop>12/17/16</Study_Stop>
        <Component_Name>ALBUMIN</Component_Name>
        <loinc_code>1810650</loinc_code>
        <Reference_Unit>g/dL</Reference_Unit>
        <Reference_Low>3.5</Reference_Low>
        <Reference_High>5.0</Reference_High>
        <Result_Value>3.9</Result_Value>
    <timestamp>1904-10-28</timestamp><redcapFormName>chemistry</redcapFormName><eventName/><formDateField/><formCompletedFieldName/><formImportedFieldName/><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
    <subject>
        <Study_Id>001-0001</Study_Id>
        <Study_Start>05/08/21</Study_Start>
        <Collection_Date>09/19/17</Collection_Date>
        <Collection_Time>11:57</Collection_Time>
        <Qualifying_Result>Y</Qualifying_Result>
        <Study_Stop>12/17/16</Study_Stop>
        <Component_Name>ALKALINE PHOSPHATASE</Component_Name>
        <loinc_code>1525848</loinc_code>
        <Reference_Unit>U/L</Reference_Unit>
        <Reference_Low>35</Reference_Low>
        <Reference_High>129</Reference_High>
        <Result_Value>112</Result_Value>
    <timestamp>1904-10-28</timestamp><redcapFormName>undefined</redcapFormName><eventName/><formDateField/><formCompletedFieldName/><formImportedFieldName/><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
    <subject>
        <Study_Id>001-0001</Study_Id>
        <Study_Start>05/08/21</Study_Start>
        <Collection_Date>09/19/17</Collection_Date>
        <Collection_Time>11:57</Collection_Time>
        <Qualifying_Result>Y</Qualifying_Result>
        <Study_Stop>12/17/16</Study_Stop>
        <Component_Name>ALPHA FETO PROT</Component_Name>
        <loinc_code>683</loinc_code>
        <Reference_Unit>ng/mL</Reference_Unit>
        <Reference_Low>0.0</Reference_Low>
        <Reference_High>8.7</Reference_High>
        <Result_Value>3.0</Result_Value>
    <timestamp>1904-10-28</timestamp><redcapFormName>undefined</redcapFormName><eventName/><formDateField/><formCompletedFieldName/><formImportedFieldName/><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
    </study>"""

        self.data = etree.ElementTree(etree.fromstring(self.sortedData))

        self.form_events = """<?xml version="1.0" encoding="UTF-8"?>
<redcapProject>
    <name>HCV 2.0</name>
    <form>
        <name>cbc</name>
        <formDateField>cbc_lbdtc</formDateField>
        <formCompletedFieldName>cbc_complete</formCompletedFieldName>
        <formImportedFieldName>cbc_nximport</formImportedFieldName>
        <event>
            <name>1_arm_1</name>
        </event>
        <event>
            <name>2_arm_1</name>
        </event>
        <event>
            <name>3_arm_1</name>
        </event>
        <event>
            <name>4_arm_1</name>
        </event>
        <event>
            <name>5_arm_1</name>
        </event>
        <event>
            <name>6_arm_1</name>
        </event>
        <event>
            <name>7_arm_1</name>
        </event>
        <event>
            <name>8_arm_1</name>
        </event>
        <event>
            <name>9_arm_1</name>
        </event>
        <event>
            <name>10_arm_1</name>
        </event>
        <event>
            <name>11_arm_1</name>
        </event>
        <event>
            <name>12_arm_1</name>
        </event>
        <event>
            <name>13_arm_1</name>
        </event>
        <event>
            <name>14_arm_1</name>
        </event>
        <event>
            <name>15_arm_1</name>
        </event>
        <event>
            <name>16_arm_1</name>
        </event>
        <event>
            <name>17_arm_1</name>
        </event>
        <event>
            <name>18_arm_1</name>
        </event>
        <event>
            <name>19_arm_1</name>
        </event>
        <event>
            <name>20_arm_1</name>
        </event>
        <event>
            <name>21_arm_1</name>
        </event>
        <event>
            <name>22_arm_1</name>
        </event>
        <event>
            <name>23_arm_1</name>
        </event>
        <event>
            <name>24_arm_1</name>
        </event>
        <event>
            <name>25_arm_1</name>
        </event>
        <event>
            <name>26_arm_1</name>
        </event>
        <event>
            <name>27_arm_1</name>
        </event>
        <event>
            <name>28_arm_1</name>
        </event>
        <event>
            <name>29_arm_1</name>
        </event>
        <event>
            <name>30_arm_1</name>
        </event>
        <event>
            <name>31_arm_1</name>
        </event>
    </form>
    <form>
        <name>chemistry</name>
        <formDateField>chem_lbdtc</formDateField>
        <formCompletedFieldName>chemistry_complete</formCompletedFieldName>
        <formImportedFieldName>chem_nximport</formImportedFieldName>
        <event>
            <name>1_arm_1</name>
        </event>
        <event>
            <name>2_arm_1</name>
        </event>
        <event>
            <name>3_arm_1</name>
        </event>
        <event>
            <name>4_arm_1</name>
        </event>
        <event>
            <name>5_arm_1</name>
        </event>
        <event>
            <name>6_arm_1</name>
        </event>
        <event>
            <name>7_arm_1</name>
        </event>
        <event>
            <name>8_arm_1</name>
        </event>
        <event>
            <name>9_arm_1</name>
        </event>
        <event>
            <name>10_arm_1</name>
        </event>
        <event>
            <name>11_arm_1</name>
        </event>
        <event>
            <name>12_arm_1</name>
        </event>
        <event>
            <name>13_arm_1</name>
        </event>
        <event>
            <name>14_arm_1</name>
        </event>
        <event>
            <name>15_arm_1</name>
        </event>
        <event>
            <name>16_arm_1</name>
        </event>
        <event>
            <name>17_arm_1</name>
        </event>
        <event>
            <name>18_arm_1</name>
        </event>
        <event>
            <name>19_arm_1</name>
        </event>
        <event>
            <name>20_arm_1</name>
        </event>
        <event>
            <name>21_arm_1</name>
        </event>
        <event>
            <name>22_arm_1</name>
        </event>
        <event>
            <name>23_arm_1</name>
        </event>
        <event>
            <name>24_arm_1</name>
        </event>
        <event>
            <name>25_arm_1</name>
        </event>
        <event>
            <name>26_arm_1</name>
        </event>
        <event>
            <name>27_arm_1</name>
        </event>
        <event>
            <name>28_arm_1</name>
        </event>
        <event>
            <name>29_arm_1</name>
        </event>
        <event>
            <name>30_arm_1</name>
        </event>
        <event>
            <name>31_arm_1</name>
        </event>
    </form>
</redcapProject>

"""

        self.form_events_tree = etree.ElementTree(etree.fromstring(self.form_events))

        self.output = """<study>
    <subject>
        <Study_Id>001-0001</Study_Id>
        <Study_Start>05/08/21</Study_Start>
        <Collection_Date>09/19/17</Collection_Date>
        <Collection_Time>11:57</Collection_Time>
        <Qualifying_Result>Y</Qualifying_Result>
        <Study_Stop>12/17/16</Study_Stop>
        <Component_Name>ALBUMIN</Component_Name>
        <loinc_code>1810650</loinc_code>
        <Reference_Unit>g/dL</Reference_Unit>
        <Reference_Low>3.5</Reference_Low>
        <Reference_High>5.0</Reference_High>
        <Result_Value>3.9</Result_Value>
    <timestamp>1904-10-28</timestamp><redcapFormName>chemistry</redcapFormName><eventName/><formDateField/><formCompletedFieldName/><formImportedFieldName>chem_nximport</formImportedFieldName><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
    <subject>
        <Study_Id>001-0001</Study_Id>
        <Study_Start>05/08/21</Study_Start>
        <Collection_Date>09/19/17</Collection_Date>
        <Collection_Time>11:57</Collection_Time>
        <Qualifying_Result>Y</Qualifying_Result>
        <Study_Stop>12/17/16</Study_Stop>
        <Component_Name>ALKALINE PHOSPHATASE</Component_Name>
        <loinc_code>1525848</loinc_code>
        <Reference_Unit>U/L</Reference_Unit>
        <Reference_Low>35</Reference_Low>
        <Reference_High>129</Reference_High>
        <Result_Value>112</Result_Value>
    <timestamp>1904-10-28</timestamp><redcapFormName>undefined</redcapFormName><eventName/><formDateField/><formCompletedFieldName/><formImportedFieldName>undefined</formImportedFieldName><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
    <subject>
        <Study_Id>001-0001</Study_Id>
        <Study_Start>05/08/21</Study_Start>
        <Collection_Date>09/19/17</Collection_Date>
        <Collection_Time>11:57</Collection_Time>
        <Qualifying_Result>Y</Qualifying_Result>
        <Study_Stop>12/17/16</Study_Stop>
        <Component_Name>ALPHA FETO PROT</Component_Name>
        <loinc_code>683</loinc_code>
        <Reference_Unit>ng/mL</Reference_Unit>
        <Reference_Low>0.0</Reference_Low>
        <Reference_High>8.7</Reference_High>
        <Result_Value>3.0</Result_Value>
    <timestamp>1904-10-28</timestamp><redcapFormName>undefined</redcapFormName><eventName/><formDateField/><formCompletedFieldName/><formImportedFieldName>undefined</formImportedFieldName><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
    </study>"""

        self.expect = etree.tostring(etree.fromstring(self.output))

    def test_update_form_imported(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        redi.update_form_imported_field(self.data, self.form_events_tree, 'undefined')
        result = etree.tostring(self.data)
        self.assertEqual(self.expect, result)

    def tearDown(self):
        return()

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import sys
import argparse
import json
from redcap import Project, RedcapError

def main():

    parser = argparse.ArgumentParser(
        description='Read some data from a REDCap Project')
    parser.add_argument(
        '--token',
        dest='token',
        default='',
        required=True,
        help='Specify the authentication/authorization token that will provide access to the REDCap project')
    parser.add_argument(
        '--url',
        dest='url',
        default='',
        required=True,
        help='Specify the url of the REDCap server to connect with')
    parser.add_argument(
        '--verify_ssl',
        dest='verify_ssl',
        default=True,
        help='Specify whether the SSL cert of the REDCap server should be checked')
    parser.add_argument('-i', '--import_data', dest='import_data', default='',
                        help='Specify the input data file to load into REDCap')
    parser.add_argument(
        '-f',
        '--forms',
        dest='forms',
        default='',
        help='Specify a list of forms, separated by spaces, for which data should be returned.')
    parser.add_argument(
        '-t',
        '--type',
        choices=['json', 'csv', 'xml'],
        dest='data_type',
        default='csv',
        help='Specify the file type used as input or output. Valid types: json, csv, xml')
    parser.add_argument(
        '--fields',
        dest='fields',
        default='',
        help='Specify a list of fields, separated by spaces, for which data should be returned.')
    parser.add_argument(
        '-e',
        '--events',
        dest='events',
        default='',
        help='Specify a list of events, separated by spaces, for which data should be returned.')
    parser.add_argument(
        '-r',
        '--records',
        dest='records',
        default='',
        help='Specify a list of records, separated by spaces, for which data should be returned.')

    # prepare the arguments we were given
    args = vars(parser.parse_args())

    # According to http://pycap.readthedocs.org/en/latest/api.html
    # allowed data_types are: csv, json, xml
    data_type = args['data_type']

    # Turn the 'verify_ssl' parameter into the truth value we need to make a
    # REDCap connection
    if args['verify_ssl'] == 'y':
        args['verify_ssl'] = True
    else:
        args['verify_ssl'] = False

    # Attempt to connect to the REDCap project
    try:
        project = Project(args['url'], args['token'], "", args['verify_ssl'])
    except:
        print "Cannot connect to project at " + args['url'] + ' with token ' + args['token']
        quit()

    # either we export data...
    if args['import_data'] == '':
        my_forms = args['forms'].split()
        my_fields = args['fields'].split()
        my_events = args['events'].split()
        my_records = args['records'].split()
        data = project.export_records(
            forms=my_forms,
            format = data_type,
            fields=my_fields,
            events=my_events,
            records=my_records,
            event_name='unique')
        if 'json' == data_type:
            print json.dumps(data, ensure_ascii=False)
        else:
            print str(data)
    else:
        # ...or we import data
        file = args['import_data']
        try:
            input = open(file, 'r')
        except IOError:
            print "Cannot open file " + file
            quit()
        if 'json' == data_type:
            json_data = json.load(input)
            response = project.import_records(json_data)
        else:
            response = project.import_records(input.read(), format = data_type)

        print response

if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import os
import logging
import shutil
import tempfile
import pysftp
from mock import patch
from redi.utils import GetEmrData
from redi.utils.GetEmrData import EmrFileAccessDetails

import time
from subprocess import Popen

# Try to silence extra info
logging.getLogger("paramico").addHandler(logging.NullHandler)
logging.getLogger("pysftp").addHandler(logging.NullHandler)

class TestGetEMRData(unittest.TestCase):

    def setUp(self):
        pass


    def _noop(*args, **kwargs):
        pass


    @patch.multiple(pysftp, Connection=_noop)
    @patch.multiple(GetEmrData, download_file=_noop)
    def test_get_emr_data(self):
        """
        This test verifies only that the csv file on the sftp server
        can be transformed to an xml file.
        Note: This test is not concerned with testing the sftp communication"""
        temp_folder = tempfile.mkdtemp('/')
        temp_txt_file = os.path.join(temp_folder, "raw.txt")
        temp_escaped_file = os.path.join(temp_folder, "rawEscaped.txt")
        temp_xml_file = os.path.join(temp_folder, "raw.xml")

        input_string = '''"NAME","COMPONENT_ID","RESULT","REFERENCE_UNIT","DATE_TIME_STAMP","STUDY_ID"
"RNA","1905","<5","IU/mL","1907-05-21 05:50:00","999-0059"
"EGFR","1740200","eGFR result is => 60 ml/min/1.73M2","ml/min/1.73M2","1903-11-27 15:13:00","999-0059"
"HEMATOCRIT","1534436",">27&<30","%","","999-0059"'''

        with open(temp_txt_file, 'w+') as f:
            f.write(input_string)

        props = EmrFileAccessDetails(
            emr_download_file='raw.csv',
            emr_host='localhost',
            emr_username='admin',
            emr_password='admin',
            emr_port='7788',
            emr_private_key=None,
            emr_private_key_pass=None,
            )

        GetEmrData.get_emr_data(temp_folder, props)
        GetEmrData.data_preprocessing(temp_txt_file, temp_escaped_file)
        GetEmrData.generate_xml(temp_escaped_file, temp_xml_file)

        with open(temp_xml_file) as f:
            result = f.read()
        expected = '''<?xml version="1.0" encoding="utf8"?>
<study>
    <subject>
        <NAME>RNA</NAME>
        <COMPONENT_ID>1905</COMPONENT_ID>
        <RESULT>&lt;5</RESULT>
        <REFERENCE_UNIT>IU/mL</REFERENCE_UNIT>
        <DATE_TIME_STAMP>1907-05-21 05:50:00</DATE_TIME_STAMP>
        <STUDY_ID>999-0059</STUDY_ID>
    </subject>
    <subject>
        <NAME>EGFR</NAME>
        <COMPONENT_ID>1740200</COMPONENT_ID>
        <RESULT>eGFR result is =&gt; 60 ml/min/1.73M2</RESULT>
        <REFERENCE_UNIT>ml/min/1.73M2</REFERENCE_UNIT>
        <DATE_TIME_STAMP>1903-11-27 15:13:00</DATE_TIME_STAMP>
        <STUDY_ID>999-0059</STUDY_ID>
    </subject>
    <subject>
        <NAME>HEMATOCRIT</NAME>
        <COMPONENT_ID>1534436</COMPONENT_ID>
        <RESULT>&gt;27&amp;&lt;30</RESULT>
        <REFERENCE_UNIT>%</REFERENCE_UNIT>
        <DATE_TIME_STAMP></DATE_TIME_STAMP>
        <STUDY_ID>999-0059</STUDY_ID>
    </subject>
</study>
'''
        self.assertEqual(result, expected)
        shutil.rmtree(temp_folder)


    def test_pysftp_using_rsa_key(self):
        """
        Starts a sftp server and transfers a file
        to verify the connection using a private key.

        Notes:
            - Disble this test if running on the Travis VM fails
                by adding one of the following annotations:
                @unittest.skip("Unconditional skipping")
                @unittest.skipIf(os.getenv('CI', '') > '', reason='Travis VM')
            - Dependency: `sudo easy_install sftpserver`
        """

        # Using a temp folder does not work for some reason
        #temp_folder = tempfile.mkdtemp('/')
        temp_folder = "."
        key_file = os.path.join(temp_folder, 'unittest_pysftp_rsa_key')
        key_file = create_rsa_key(key_file)

        source_file = os.path.join(temp_folder, 'source_file')
        source_file = create_sample_file(source_file)

        # At this point the destination file is empty
        destination_file = os.path.join(temp_folder, 'destination_file')
        proc = None

        try:
            sftp_cmd = "sftpserver --host localhost -p 7788 -l DEBUG -k " + key_file
            proc = Popen(sftp_cmd, shell=True)
            time.sleep(1) # let the server start

            try:
                # this block depends on a running sftp server
                connection_info = get_connection_info(key_file)
                connection_info['port'] = int(connection_info['port'])
                with pysftp.Connection(**connection_info) as sftp:
                    logging.info("User %s connected to sftp server %s" % \
                        (connection_info['username'], connection_info['host']))
                    #print sftp.listdir('.')
                    sftp.get(source_file, destination_file)
            except Exception as e:
                logging.error("Problem connecting to: %s due error: %s" % \
                        (connection_info['host'], e))
        except Exception as e:
            logging.error("Problem starting sftp server due error: %s for file: %s" % \
                    (e, destination_file))
        finally:
            if proc:
                proc.terminate()

        with open(destination_file) as fresh_file:
            actual = fresh_file.read()
            self.assertEqual(actual, "SFTP TEST")

        # Cleanup created files
        os.remove(source_file)
        os.remove(destination_file)
        os.remove(key_file)
        os.remove(key_file +".pub")


def create_rsa_key(rsa_key_file):
    """Create a RSA private key pair to be used by sftp"""
    if not os.path.isfile(rsa_key_file):
        cmd = "ssh-keygen -q -t rsa -N '' -f " + rsa_key_file
        proc = Popen(cmd, shell=True)
        time.sleep(1)
        proc.terminate()
    else:
        logging.warn("RSA key file already exists: %s" % rsa_key_file)

    return rsa_key_file


def create_sample_file(sample_file):
    """ Create a sample file to be transfered ofe sftp"""
    if not os.path.isfile(sample_file):
        try:
            f = open(sample_file, 'w+')
            f.write("SFTP TEST")
            f.flush()
            f.close()
        except IOError as (errno, strerror):
            logging.error("I/O error({0}): {1}".format(errno, strerror))
    else:
        logging.info("Sample file %s already exists" % sample_file)
    return sample_file


def get_connection_info(private_key):
    """Return a dictionary of parameters for creating a sftp connection"""
    access_details = EmrFileAccessDetails(
        emr_download_file='raw.csv',
        emr_host='localhost',
        emr_username='admin',
        emr_password='admin',
        emr_port='7788',
        emr_private_key=private_key,
        emr_private_key_pass=None,
    )

    connection_info = dict(access_details.__dict__)
    # delete unnecessary element form the dictionary
    del connection_info['download_file']
    return connection_info


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import os
import sys
from lxml import etree
from StringIO import StringIO
import time
from redi import redi
from redi import report

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestCreateSummaryReport(unittest.TestCase):
    """
    Unit test for `redi.create_summary_report()`
    """

    def setUp(self):
        """
        Prepare data structures
        """
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.test_report_params = {
            'project': 'hcvtarget-uf',
            'report_file_path': os.path.join(DEFAULT_DATA_DIRECTORY, 'unittest_report.xml'),
            'redcap_uri': 'https://hostname.org',
            'is_sort_by_lab_id': True,
            }

        self.test_report_data = {
            'total_subjects': 5,
            'form_details': {
                'Total_chemistry_Forms': 22,
                'Total_cbc_Forms': 53
            },
            'subject_details': {
                '60': {'cbc_Forms': 1, 'chemistry_Forms': 1, 'lab_id': '999-0060'},
                '61': {'cbc_Forms': 2, 'chemistry_Forms': 1, 'lab_id': '999-0061'},
                '63': {'cbc_Forms': 11, 'chemistry_Forms': 4, 'lab_id': '999-0063'},
                '59': {'cbc_Forms': 39, 'chemistry_Forms': 16, 'lab_id': '999-0059'}
            },
            'errors' : [],
        }

        self.test_alert_summary = {
            'multiple_values_alert': [
                'This is multiple values alert 1',
                'This is multiple values alert 2',
                'This is multiple values alert 3'],
            'max_event_alert': [
                'This is max event alert 1',
                'This is max event alert 2',
                'This is max event alert 3']
        }

        self.specimen_taken_time_summary = {'total': 15, 'blank': 3}
        self.duration_dict = {
            'all' : {
                'start': "2014-01-01 00:00:00",
                'end': "2014-01-01 00:00:01",
            }
        }

        self.expected_xml = '''
<report>
    <header>
        <project>hcvtarget-uf</project>
        <date>'''+time.strftime("%m/%d/%Y")+'''</date>
        <redcapServerAddress>https://hostname.org</redcapServerAddress>
    </header>
    <summary>
        <subjectCount>5</subjectCount>
        <forms>
            <form>
                <form_name>Total_cbc_Forms</form_name>
                <form_count>53</form_count>
            </form>
            <form>
                <form_name>Total_chemistry_Forms</form_name>
                <form_count>22</form_count>
            </form>
        </forms>
    </summary>
    <alerts>
        <tooManyForms>
        <eventAlert>
            <message>This is max event alert 1</message>
        </eventAlert>
        <eventAlert>
            <message>This is max event alert 2</message>
        </eventAlert>
        <eventAlert>
            <message>This is max event alert 3</message>
        </eventAlert>
        </tooManyForms>
        <tooManyValues>
            <valuesAlert>
                <message>This is multiple values alert 1</message>
            </valuesAlert>
            <valuesAlert>
                <message>This is multiple values alert 2</message>
            </valuesAlert>
            <valuesAlert><message>This is multiple values alert 3</message>
        </valuesAlert></tooManyValues>
    </alerts>
    <subjectsDetails>
        <subject>
        <redcap_id>59</redcap_id>
        <forms>
            <form>
                <form_name>cbc_Forms</form_name>
                <form_count>39</form_count>
            </form>
            <form>
                <form_name>chemistry_Forms</form_name>
                <form_count>16</form_count>
            </form>
        </forms>
        <lab_id>999-0059</lab_id>
        </subject>
        <subject>
            <redcap_id>60</redcap_id>
            <forms>
                <form>
                    <form_name>cbc_Forms</form_name>
                    <form_count>1</form_count></form>
                <form>
                    <form_name>chemistry_Forms</form_name>
                    <form_count>1</form_count>
                </form>
            </forms>
            <lab_id>999-0060</lab_id>
        </subject>
        <subject>
            <redcap_id>61</redcap_id>
            <forms>
                <form>
                    <form_name>cbc_Forms</form_name>
                    <form_count>2</form_count>
                </form>
                <form>
                    <form_name>chemistry_Forms</form_name>
                    <form_count>1</form_count>
                </form>
            </forms>
            <lab_id>999-0061</lab_id>
        </subject>
        <subject>
            <redcap_id>63</redcap_id>
            <forms>
                <form>
                    <form_name>cbc_Forms</form_name>
                    <form_count>11</form_count>
                </form>
                <form>
                    <form_name>chemistry_Forms</form_name>
                    <form_count>4</form_count>
                </form>
            </forms>
            <lab_id>999-0063</lab_id>
        </subject>
    </subjectsDetails>
    <errors/>
    <summaryOfSpecimenTakenTimes>
        <total>15</total>
        <blank>3</blank>
        <percent>20.0</percent>
    </summaryOfSpecimenTakenTimes>
    <sort_details_by>lab_id</sort_details_by>
    <time_all_start>00:00:00</time_all_start>
    <time_all_end>00:00:01</time_all_end>
    <time_all_diff>0:00:01</time_all_diff>
</report>'''

        self.schema_str = StringIO('''\
            <xs:schema attributeFormDefault="unqualified" elementFormDefault="qualified" xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="report">
    <xs:complexType>
      <xs:sequence>
        <xs:element name="header">
          <xs:complexType>
            <xs:sequence>
              <xs:element type="xs:string" name="project"/>
              <xs:element type="xs:string" name="date"/>
              <xs:element type="xs:string" name="redcapServerAddress"/>
            </xs:sequence>
          </xs:complexType>
        </xs:element>
        <xs:element name="summary">
          <xs:complexType>
            <xs:sequence>
              <xs:element type="xs:byte" name="subjectCount"/>
              <xs:element name="forms">
                <xs:complexType>
                  <xs:sequence>
                    <xs:element name="form" maxOccurs="unbounded" minOccurs="0">
                      <xs:complexType>
                        <xs:sequence>
                          <xs:element type="xs:string" name="form_name"/>
                          <xs:element type="xs:byte" name="form_count"/>
                        </xs:sequence>
                      </xs:complexType>
                    </xs:element>
                  </xs:sequence>
                </xs:complexType>
              </xs:element>
            </xs:sequence>
          </xs:complexType>
        </xs:element>
        <xs:element name="alerts">
          <xs:complexType>

             <xs:sequence>
              <xs:element name="tooManyForms">
                <xs:complexType>
                  <xs:sequence>
                    <xs:element name="eventAlert" maxOccurs="unbounded" minOccurs="0">
                      <xs:complexType>
                        <xs:sequence>
                          <xs:element type="xs:string" name="message"/>
                        </xs:sequence>
                      </xs:complexType>
                    </xs:element>
                  </xs:sequence>
                </xs:complexType>
              </xs:element>

              <xs:element name="tooManyValues">
                <xs:complexType>
                  <xs:sequence>
                    <xs:element name="valuesAlert" maxOccurs="unbounded" minOccurs="0">
                      <xs:complexType>
                        <xs:sequence>
                          <xs:element type="xs:string" name="message"/>
                        </xs:sequence>
                      </xs:complexType>
                    </xs:element>
                  </xs:sequence>
                </xs:complexType>
              </xs:element>
            </xs:sequence>
          </xs:complexType>
        </xs:element>
        <xs:element name="subjectsDetails">
          <xs:complexType>
            <xs:sequence>
              <xs:element name="subject" maxOccurs="unbounded" minOccurs="0">
                <xs:complexType>
                  <xs:sequence>
                    <xs:element type="xs:int" name="redcap_id"/>
                    <xs:element name="forms">
                      <xs:complexType>
                        <xs:sequence>
                          <xs:element name="form" maxOccurs="unbounded" minOccurs="0">
                            <xs:complexType>
                              <xs:sequence>
                                <xs:element type="xs:string" name="form_name"/>
                                <xs:element type="xs:byte" name="form_count"/>
                              </xs:sequence>
                            </xs:complexType>
                          </xs:element>
                        </xs:sequence>
                      </xs:complexType>
                    </xs:element>
                    <xs:element type="xs:string" name="lab_id"/>
                  </xs:sequence>
                </xs:complexType>
              </xs:element>
            </xs:sequence>
          </xs:complexType>
        </xs:element>
        <xs:element name="errors">
        </xs:element>
        <xs:element name="summaryOfSpecimenTakenTimes">
          <xs:complexType>
            <xs:sequence>
              <xs:element type="xs:byte" name="total"/>
              <xs:element type="xs:byte" name="blank"/>
              <xs:element type="xs:float" name="percent"/>
            </xs:sequence>
          </xs:complexType>
        </xs:element>
        <xs:element name="sort_details_by"></xs:element>
        <xs:element name="time_all_start"></xs:element>
        <xs:element name="time_all_end"></xs:element>
        <xs:element name="time_all_diff"></xs:element>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>''')
        return

    def test_create_summary_report(self):
        """
        Validates the summary xml structure using xsd
        Validate the summary xml content
        """
        sys.path.append('config')

        class MockWriter(object):
            def __call__(self, *args, **kwargs):
                #expected call: write(tree, report_file_path)
                self.result = args[0]
        writer = MockWriter()

        creator = report.ReportCreator(
            self.test_report_params['report_file_path'],
            self.test_report_params['project'],
            self.test_report_params['redcap_uri'],
            self.test_report_params['is_sort_by_lab_id'],
            writer)

        creator.create_report(self.test_report_data, self.test_alert_summary,
                              self.specimen_taken_time_summary, self.duration_dict)

        result = writer.result
        result_string = etree.tostring(result)
        #print result_string
        xmlschema_doc = etree.parse(self.schema_str)
        xml_schema = etree.XMLSchema(xmlschema_doc)
        # validate the xml against the xsd schema
        self.assertEqual(xml_schema.validate(result), True)

        # validate the actual data in xml but strip the white space first
        parser = etree.XMLParser(remove_blank_text=True)
        clean_tree = etree.XML(self.expected_xml, parser=parser)
        self.expected_xml = etree.tostring(clean_tree)
        self.assertEqual(self.expected_xml, result_string)

    def tearDown(self):
        # delete the created xml file
        try:
            os.remove(self.test_report_params['report_file_path'])
        except:
            pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
from lxml import etree
import os
from redi import redi

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestUpdateStatusField(unittest.TestCase):

  def setUp(self):
    redi.configure_logging(DEFAULT_DATA_DIRECTORY)


  def test_update_status_field_value_when_one_subject_with_two_forms_with_one_event_in_each_form(self):

    self.source = """<person_form_event>
      <person>
          <study_id>99</study_id>
          <all_form_events>
              <form>
                  <name>cbc</name>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>cbc_lbdtc</name>
                          <value>1905-10-01</value>
                      </field>
                      <field>
                          <name>wbc_lborres</name>
                          <value>3.0</value>
                      </field>
                      <field>
                          <name>wbc_lbstat</name>
                          <value/>
                      </field>
                      <field>
                          <name>neut_lborres</name>
                          <value>500</value>
                      </field>
                      <field>
                          <name>neut_lbstat</name>
                          <value/>
                      </field>
                  </event>
              </form>
              <form>
                  <name>inr</name>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>inr_lbdtc</name>
                          <value>1906-12-01</value>
                      </field>
                      <field>
                          <name>inr_lborres</name>
                          <value/>
                      </field>
                      <field>
                          <name>inr_lbstat</name>
                          <value/>
                      </field>
                  </event>
              </form>
          </all_form_events>
      </person>
      </person_form_event>
      """

    self.source_tree = etree.ElementTree(etree.fromstring(self.source))

    self.input = """
    <rediFieldMap>
      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>wbc_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>wbc_lborresu</redcapFieldNameUnits>
          <redcapStatusFieldName>wbc_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>neut_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>neut_lborresu</redcapFieldNameUnits>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>inr</redcapFormName>
          <redcapFieldNameValue>inr_lborres</redcapFieldNameValue>
          <redcapStatusFieldName>inr_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>
      </rediFieldMap>
    """

    self.input_tree = etree.ElementTree(etree.fromstring(self.input))

    self.output = """<person_form_event>
      <person>
          <study_id>99</study_id>
          <all_form_events>
              <form>
                  <name>cbc</name>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>cbc_lbdtc</name>
                          <value>1905-10-01</value>
                      </field>
                      <field>
                          <name>wbc_lborres</name>
                          <value>3.0</value>
                      </field>
                      <field>
                          <name>wbc_lbstat</name>
                          <value/>
                      </field>
                      <field>
                          <name>neut_lborres</name>
                          <value>500</value>
                      </field>
                      <field>
                          <name>neut_lbstat</name>
                          <value/>
                      </field>
                  </event>
              </form>
              <form>
                  <name>inr</name>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>inr_lbdtc</name>
                          <value>1906-12-01</value>
                      </field>
                      <field>
                          <name>inr_lborres</name>
                          <value/>
                      </field>
                      <field>
                          <name>inr_lbstat</name>
                          <value>NOT_DONE</value>
                      </field>
                  </event>
              </form>
          </all_form_events>
      </person>
      </person_form_event>
    """
    self.expect = etree.tostring(etree.fromstring(self.output))
    redi.updateStatusFieldValueInPersonFormEventTree(self.source_tree, self.input_tree)
    result = etree.tostring(self.source_tree)
    self.assertEqual(self.expect, result)

  def test_update_status_field_value_when_one_subject_with_two_forms_with_two_events_in_each_form(self):
    self.source = """<person_form_event>
      <person>
          <study_id>99</study_id>
          <all_form_events>
              <form>
                  <name>cbc</name>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>cbc_lbdtc</name>
                          <value>1905-10-01</value>
                      </field>
                      <field>
                          <name>wbc_lborres</name>
                          <value>3.0</value>
                      </field>
                      <field>
                          <name>wbc_lbstat</name>
                          <value/>
                      </field>
                      <field>
                          <name>neut_lborres</name>
                          <value>500</value>
                      </field>
                      <field>
                          <name>neut_lbstat</name>
                          <value/>
                      </field>
                  </event>
                  <event>
                      <name>1_arm_2</name>
                      <field>
                          <name>cbc_lbdtc</name>
                          <value>1905-10-01</value>
                      </field>
                      <field>
                          <name>wbc_lborres</name>
                          <value>3.0</value>
                      </field>
                      <field>
                          <name>wbc_lbstat</name>
                          <value/>
                      </field>
                      <field>
                          <name>neut_lborres</name>
                          <value/>
                      </field>
                      <field>
                          <name>neut_lbstat</name>
                          <value/>
                      </field>
                  </event>
              </form>
              <form>
                  <name>inr</name>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>inr_lbdtc</name>
                          <value>1906-12-01</value>
                      </field>
                      <field>
                          <name>inr_lborres</name>
                          <value/>
                      </field>
                      <field>
                          <name>inr_lbstat</name>
                          <value/>
                      </field>
                  </event>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>inr_lbdtc</name>
                          <value>1906-12-01</value>
                      </field>
                      <field>
                          <name>inr_lborres</name>
                          <value>2</value>
                      </field>
                      <field>
                          <name>inr_lbstat</name>
                          <value/>
                      </field>
                  </event>
              </form>
          </all_form_events>
      </person>
      </person_form_event>
      """

    self.source_tree = etree.ElementTree(etree.fromstring(self.source))

    self.input = """
    <rediFieldMap>
      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>wbc_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>wbc_lborresu</redcapFieldNameUnits>
          <redcapStatusFieldName>wbc_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>neut_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>neut_lborresu</redcapFieldNameUnits>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>inr</redcapFormName>
          <redcapFieldNameValue>inr_lborres</redcapFieldNameValue>
          <redcapStatusFieldName>inr_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>
      </rediFieldMap>
    """

    self.input_tree = etree.ElementTree(etree.fromstring(self.input))

    self.output = """<person_form_event>
      <person>
          <study_id>99</study_id>
          <all_form_events>
              <form>
                  <name>cbc</name>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>cbc_lbdtc</name>
                          <value>1905-10-01</value>
                      </field>
                      <field>
                          <name>wbc_lborres</name>
                          <value>3.0</value>
                      </field>
                      <field>
                          <name>wbc_lbstat</name>
                          <value />
                      </field>
                      <field>
                          <name>neut_lborres</name>
                          <value>500</value>
                      </field>
                      <field>
                          <name>neut_lbstat</name>
                          <value />
                      </field>
                  </event>
                  <event>
                      <name>1_arm_2</name>
                      <field>
                          <name>cbc_lbdtc</name>
                          <value>1905-10-01</value>
                      </field>
                      <field>
                          <name>wbc_lborres</name>
                          <value>3.0</value>
                      </field>
                      <field>
                          <name>wbc_lbstat</name>
                          <value />
                      </field>
                      <field>
                          <name>neut_lborres</name>
                          <value />
                      </field>
                      <field>
                          <name>neut_lbstat</name>
                          <value />
                      </field>
                  </event>
              </form>
              <form>
                  <name>inr</name>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>inr_lbdtc</name>
                          <value>1906-12-01</value>
                      </field>
                      <field>
                          <name>inr_lborres</name>
                          <value />
                      </field>
                      <field>
                          <name>inr_lbstat</name>
                          <value>NOT_DONE</value>
                      </field>
                  </event>
                  <event>
                      <name>1_arm_1</name>
                      <field>
                          <name>inr_lbdtc</name>
                          <value>1906-12-01</value>
                      </field>
                      <field>
                          <name>inr_lborres</name>
                          <value>2</value>
                      </field>
                      <field>
                          <name>inr_lbstat</name>
                          <value />
                      </field>
                  </event>
              </form>
          </all_form_events>
      </person>
      </person_form_event>
    """
    self.expect = etree.tostring(etree.fromstring(self.output))
    redi.updateStatusFieldValueInPersonFormEventTree(self.source_tree, self.input_tree)
    result = etree.tostring(self.source_tree)
    self.assertEqual(self.expect, result)

  def test_update_status_field_value_when_two_subjects_with_two_forms_with_one_event_in_each_form(self):
    self.source = """<person_form_event>
          <person>
              <study_id>99</study_id>
              <all_form_events>
                  <form>
                      <name>cbc</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>cbc_lbdtc</name>
                              <value>1905-10-01</value>
                          </field>
                          <field>
                              <name>wbc_lborres</name>
                              <value>3.0</value>
                          </field>
                          <field>
                              <name>wbc_lbstat</name>
                              <value/>
                          </field>
                          <field>
                              <name>neut_lborres</name>
                              <value>500</value>
                          </field>
                          <field>
                              <name>neut_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
                  <form>
                      <name>inr</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>inr_lbdtc</name>
                              <value>1906-12-01</value>
                          </field>
                          <field>
                              <name>inr_lborres</name>
                              <value/>
                          </field>
                          <field>
                              <name>inr_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
              </all_form_events>
          </person>
          <person>
              <study_id>99</study_id>
              <all_form_events>
                  <form>
                      <name>cbc</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>cbc_lbdtc</name>
                              <value>1905-10-01</value>
                          </field>
                          <field>
                              <name>wbc_lborres</name>
                              <value/>
                          </field>
                          <field>
                              <name>wbc_lbstat</name>
                              <value/>
                          </field>
                          <field>
                              <name>neut_lborres</name>
                              <value>500</value>
                          </field>
                          <field>
                              <name>neut_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
                  <form>
                      <name>inr</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>inr_lbdtc</name>
                              <value>1906-12-01</value>
                          </field>
                          <field>
                              <name>inr_lborres</name>
                              <value>2</value>
                          </field>
                          <field>
                              <name>inr_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
              </all_form_events>
          </person>
      </person_form_event>
      """

    self.source_tree = etree.ElementTree(etree.fromstring(self.source))

    self.input = """
    <rediFieldMap>
      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>wbc_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>wbc_lborresu</redcapFieldNameUnits>
          <redcapStatusFieldName>wbc_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>neut_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>neut_lborresu</redcapFieldNameUnits>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>inr</redcapFormName>
          <redcapFieldNameValue>inr_lborres</redcapFieldNameValue>
          <redcapStatusFieldName>inr_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>
      </rediFieldMap>
    """

    self.input_tree = etree.ElementTree(etree.fromstring(self.input))

    self.output = """<person_form_event>
          <person>
              <study_id>99</study_id>
              <all_form_events>
                  <form>
                      <name>cbc</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>cbc_lbdtc</name>
                              <value>1905-10-01</value>
                          </field>
                          <field>
                              <name>wbc_lborres</name>
                              <value>3.0</value>
                          </field>
                          <field>
                              <name>wbc_lbstat</name>
                              <value/>
                          </field>
                          <field>
                              <name>neut_lborres</name>
                              <value>500</value>
                          </field>
                          <field>
                              <name>neut_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
                  <form>
                      <name>inr</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>inr_lbdtc</name>
                              <value>1906-12-01</value>
                          </field>
                          <field>
                              <name>inr_lborres</name>
                              <value/>
                          </field>
                          <field>
                              <name>inr_lbstat</name>
                              <value>NOT_DONE</value>
                          </field>
                      </event>
                  </form>
              </all_form_events>
          </person>
          <person>
              <study_id>99</study_id>
              <all_form_events>
                  <form>
                      <name>cbc</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>cbc_lbdtc</name>
                              <value>1905-10-01</value>
                          </field>
                          <field>
                              <name>wbc_lborres</name>
                              <value/>
                          </field>
                          <field>
                              <name>wbc_lbstat</name>
                              <value>NOT_DONE</value>
                          </field>
                          <field>
                              <name>neut_lborres</name>
                              <value>500</value>
                          </field>
                          <field>
                              <name>neut_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
                  <form>
                      <name>inr</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>inr_lbdtc</name>
                              <value>1906-12-01</value>
                          </field>
                          <field>
                              <name>inr_lborres</name>
                              <value>2</value>
                          </field>
                          <field>
                              <name>inr_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
              </all_form_events>
          </person>
      </person_form_event>
    """
    self.expect = etree.tostring(etree.fromstring(self.output))
    redi.updateStatusFieldValueInPersonFormEventTree(self.source_tree, self.input_tree)
    result = etree.tostring(self.source_tree)
    self.assertEqual(self.expect, result)

  def test_update_status_field_value_when_one_subject_with_no_form(self):
    self.source = """<person_form_event>
          <person>
              <study_id>99</study_id>
              <all_form_events>
              </all_form_events>
          </person>
      </person_form_event>
      """

    self.source_tree = etree.ElementTree(etree.fromstring(self.source))

    self.input = """
    <rediFieldMap>
      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>wbc_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>wbc_lborresu</redcapFieldNameUnits>
          <redcapStatusFieldName>wbc_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>neut_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>neut_lborresu</redcapFieldNameUnits>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>inr</redcapFormName>
          <redcapFieldNameValue>inr_lborres</redcapFieldNameValue>
          <redcapStatusFieldName>inr_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>
      </rediFieldMap>
    """

    self.input_tree = etree.ElementTree(etree.fromstring(self.input))

    self.output = """<person_form_event>
          <person>
              <study_id>99</study_id>
              <all_form_events>
              </all_form_events>
          </person>
      </person_form_event>
    """
    self.expect = etree.tostring(etree.fromstring(self.output))
    redi.updateStatusFieldValueInPersonFormEventTree(self.source_tree, self.input_tree)
    result = etree.tostring(self.source_tree)
    self.assertEqual(self.expect, result)

  def test_update_status_field_value_when_one_subject_with_two_forms_event_missing_in_one_of_the_forms(self):
    self.source = """<person_form_event>
          <person>
              <study_id>99</study_id>
              <all_form_events>
                  <form>
                      <name>cbc</name>
                  </form>
                  <form>
                      <name>inr</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>inr_lbdtc</name>
                              <value>1906-12-01</value>
                          </field>
                          <field>
                              <name>inr_lborres</name>
                              <value/>
                          </field>
                          <field>
                              <name>inr_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
              </all_form_events>
          </person>
      </person_form_event>
      """

    self.source_tree = etree.ElementTree(etree.fromstring(self.source))

    self.input = """
    <rediFieldMap>
      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>wbc_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>wbc_lborresu</redcapFieldNameUnits>
          <redcapStatusFieldName>wbc_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>neut_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>neut_lborresu</redcapFieldNameUnits>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>inr</redcapFormName>
          <redcapFieldNameValue>inr_lborres</redcapFieldNameValue>
          <redcapStatusFieldName>inr_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>
      </rediFieldMap>
    """

    self.input_tree = etree.ElementTree(etree.fromstring(self.input))

    self.output = """<person_form_event>
          <person>
              <study_id>99</study_id>
              <all_form_events>
                  <form>
                      <name>cbc</name>
                  </form>
                  <form>
                      <name>inr</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>inr_lbdtc</name>
                              <value>1906-12-01</value>
                          </field>
                          <field>
                              <name>inr_lborres</name>
                              <value/>
                          </field>
                          <field>
                              <name>inr_lbstat</name>
                              <value>NOT_DONE</value>
                          </field>
                      </event>
                  </form>
              </all_form_events>
          </person>
      </person_form_event>
    """
    self.expect = etree.tostring(etree.fromstring(self.output))
    redi.updateStatusFieldValueInPersonFormEventTree(self.source_tree, self.input_tree)
    result = etree.tostring(self.source_tree)
    self.assertEqual(self.expect, result)

  def test_update_status_field_value_when_one_subject_with_one_form_one_event_value_tag_missing(self):
    self.source = """<person_form_event>
          <person>
              <study_id>99</study_id>
              <all_form_events>
                  <form>
                      <name>inr</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>inr_lbdtc</name>
                              <value>1906-12-01</value>
                          </field>
                          <field>
                              <name>inr_lborres</name>
                          </field>
                          <field>
                              <name>inr_lbstat</name>
                              <value/>
                          </field>
                      </event>
                  </form>
              </all_form_events>
          </person>
      </person_form_event>
      """

    self.source_tree = etree.ElementTree(etree.fromstring(self.source))

    self.input = """
    <rediFieldMap>
      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>wbc_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>wbc_lborresu</redcapFieldNameUnits>
          <redcapStatusFieldName>wbc_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>cbc</redcapFormName>
          <redcapFieldNameValue>neut_lborres</redcapFieldNameValue>
          <redcapFieldNameUnits>neut_lborresu</redcapFieldNameUnits>
      </clinicalComponent>

      <clinicalComponent>
          <redcapFormName>inr</redcapFormName>
          <redcapFieldNameValue>inr_lborres</redcapFieldNameValue>
          <redcapStatusFieldName>inr_lbstat</redcapStatusFieldName>
          <redcapStatusFieldValue>NOT_DONE</redcapStatusFieldValue>
      </clinicalComponent>
      </rediFieldMap>
    """

    self.input_tree = etree.ElementTree(etree.fromstring(self.input))

    self.output = """<person_form_event>
          <person>
              <study_id>99</study_id>
              <all_form_events>
                  <form>
                      <name>inr</name>
                      <event>
                          <name>1_arm_1</name>
                          <field>
                              <name>inr_lbdtc</name>
                              <value>1906-12-01</value>
                          </field>
                          <field>
                              <name>inr_lborres</name>
                          </field>
                          <field>
                              <name>inr_lbstat</name>
                              <value>NOT_DONE</value>
                          </field>
                      </event>
                  </form>
              </all_form_events>
          </person>
      </person_form_event>
    """
    self.expect = etree.tostring(etree.fromstring(self.output))
    redi.updateStatusFieldValueInPersonFormEventTree(self.source_tree, self.input_tree)
    result = etree.tostring(self.source_tree)
    self.assertEqual(self.expect, result)

  def tearDown(self):
    return()

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
@author : Radha
email : rkandula@ufl.edu

This file tests if the `configure_logging`
properly creates a log file

Note: the created file is destroyed at the end
'''

import unittest
import os
import sys
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestLog(unittest.TestCase):

    def setUp(self):
        # initialize log file name
        self.file_name = proj_root + 'log/redi.log'

    def test_log(self):
        import os.path
        file_name = self.file_name
        sys.path.append('log')
        # remove any existing log file in log/ folder

        #if os.path.isfile(file_name):
        #    with open(file_name):
                #print "here"
        #        os.remove(file_name)
                #print os.path.isfile(file_name)

        # call the configure logging function
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)

        #print 'checking if log file was created: ' + file_name
        # check if the file is created
        assert os.path.isfile(file_name) == True

        # remove the file created through testing
        os.remove(file_name)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
from lxml import etree
from form import Form


class TestEventIsEmpty(unittest.TestCase):
    def test_is_empty(self):
        raw_xml = """
            <person_form_event>
              <person>
                <study_id>60</study_id>
                <all_form_events>
                  <form>
                    <name>hcv_rna_results</name>
                    <event>
                      <name>1_arm_1</name>
                      <field><name/><value/></field>
                      <field><name>hcv_lbdtc</name><value/></field>
                    </event>
                  </form>
                </all_form_events>
              </person>
            </person_form_event>"""

        form = Form(etree.fromstring(raw_xml))
        event = form.events().next()

        self.assertTrue(event.is_empty())

    def test_is_not_empty(self):
        raw_xml = """
            <person_form_event>
              <person>
                <study_id>60</study_id>
                <all_form_events>
                  <form>
                    <name>hcv_rna_results</name>
                    <event>
                      <name>1_arm_1</name>
                      <field><name/><value>o hai</value></field>
                      <field><name>hcv_lbdtc</name><value/></field>
                    </event>
                  </form>
                </all_form_events>
              </person>
            </person_form_event>"""

        form = Form(etree.fromstring(raw_xml))
        event = form.events().next()

        self.assertFalse(event.is_empty())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import smtplib
from mock import patch, call
from redi import redi
from redi.utils.rawxml import RawXml
from redi.utils import redi_email


class TestRediEmail(unittest.TestCase):
    """
    Check functions in the `utils/redi_email` module
    To run individually:
        $ PYTHONPATH=redi python test/TestRediEmail.py
    """

    def setUp(self):
        """ Prepare the settings objects"""
        settings = {
            'smtp_host_for_outbound_mail': "a.com",
            'smtp_port_for_outbound_mail': 1234,
            'redcap_support_sender_email': "support@example.com",
            'redcap_support_receiver_email': "a@example.com b@example.com",
            'redcap_uri': "http://example.com",
            'batch_warning_days': 13,
            'sender_email': 'report_sender@example.com',
            'receiver_email': 'rep_a@example.com rep_b@example.com',
            'project': 'TEST SITE',
            }
        self.settings = type("", (), settings)()
        self.email_settings = redi.get_email_settings(self.settings)
        self.raw_xml = RawXml('', __file__)


    def test_get_email_settings(self):
        """Check if we picked proper values from the global settings"""
        ese = self.email_settings

        self.assertEqual(ese['smtp_host_for_outbound_mail'], "a.com")
        self.assertEqual(ese['smtp_port_for_outbound_mail'], 1234)
        self.assertEqual(ese['redcap_support_sender_email'], \
                "support@example.com")
        self.assertEqual(ese['redcap_support_receiving_list'], \
                ["a@example.com", "b@example.com"])
        self.assertEqual(ese['redcap_uri'], "http://example.com")
        self.assertEqual(ese['batch_warning_days'], 13)
        self.assertEqual(ese['batch_report_sender_email'], \
                "report_sender@example.com")
        self.assertEqual(ese['batch_report_receiving_list'], \
                ["rep_a@example.com", "rep_b@example.com"])
        self.assertEqual(ese['site_name'], "TEST SITE")

    def dummy_send_success(*args, **kwargs):
        """Skip sending email"""
        return True

    def dummy_send_failed(*args, **kwargs):
        """Skip sending email"""
        return False

    @patch.multiple(redi_email, send_email=dummy_send_success)
    def test_success(self):
        """ Verify return true when email is sent"""
        ese = self.email_settings
        self.assertTrue(redi_email.send_email_redcap_connection_error(ese))
        self.assertTrue(redi_email.send_email_input_data_unchanged(ese, self.raw_xml))

    @patch.multiple(redi_email, send_email=dummy_send_failed)
    def test_failed(self):
        """ Verify return false when email is not sent"""
        ese = self.email_settings
        self.assertFalse(redi_email.send_email_redcap_connection_error(ese))
        self.assertFalse(redi_email.send_email_input_data_unchanged(ese, self.raw_xml))

    @patch("smtplib.SMTP")
    def test_mime_email(self, mock_smtp):
        """ Mock the sendmail function """
        ese = self.email_settings
        instance = mock_smtp.return_value
        instance.sendmail.return_value = {}
        refused_list = redi_email.send_email_data_import_completed(ese)
        self.assertIsInstance(refused_list, dict)
        self.assertEqual(refused_list, {})

    @patch("smtplib.SMTP")
    def test_mime_email_exception(self, mock_smtp):
        """ Mock the sendmail exception """
        ese = self.email_settings
        instance = mock_smtp.return_value
        instance.sendmail.side_effect = smtplib.SMTPRecipientsRefused({})
        self.assertRaises(smtplib.SMTPRecipientsRefused,\
                redi_email.send_email_data_import_completed, ese)
        self.assertEqual(instance.sendmail.call_count, 1)

    def tearDown(self):
        """ Remove dictionary"""
        self.email_settings.clear()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
# DO NOT IMPORT redi here!
# Each test needs it's own redi module so we can manipulate the
# module-scoped variables and functions


class TestResume(unittest.TestCase):

    def test_no_resume_deletes_old_run_data(self):
        class MockPersonFormEvents(object):
            def delete(self):
                raise FileDeleted()

        class FileDeleted():
            pass

        import redi.redi
        redi_ref = reload(redi.redi)
        redi_ref._person_form_events_service = MockPersonFormEvents()

        import redi.batch
        batch = reload(redi.batch)
        batch.check_input_file = lambda *args: None

        with self.assertRaises(FileDeleted):
            redi_ref._run(config_file=None, configuration_directory='',
                      do_keep_gen_files=None, dry_run=True, get_emr_data=False,
                      settings=MockSettings(), data_folder=None,
                      database_path=None, redcap_client=None,
                      report_courier=None, report_creator=None)

    def test_no_resume_stores(self):
        class MockPersonFormEvents(object):
            def delete(self):
                pass

            def store(self, ignored):
                raise FileStored()

        class FileStored():
            pass

        import redi.redi
        redi_ref = reload(redi.redi)

        redi_ref._person_form_events_service = MockPersonFormEvents()
        redi_ref._create_person_form_event_tree_with_data = lambda *args: (
            None, None, None, None, None)
        redi_ref._delete_last_runs_data = lambda *args: None
        redi_ref._removedirs = lambda *args: None
        redi_ref._mkdir = lambda *args: None
        redi_ref.connect_to_redcap = lambda *args: None

        import redi.batch
        batch = reload(redi.batch)
        batch.check_input_file = lambda *args: None

        with self.assertRaises(FileStored):
            redi_ref._run(config_file=None, configuration_directory='',
                      do_keep_gen_files=None, dry_run=True, get_emr_data=False,
                      settings=MockSettings(), data_folder=None,
                      database_path=None, redcap_client=None,
                      report_courier=None, report_creator=None)

    def test_resume_fetches_data_from_last_run(self):
        class MockPersonFormEvents(object):
            def fetch(self):
                raise DataFetched()

        class DataFetched():
            pass

        import redi.redi
        redi_ref = reload(redi.redi)
        redi_ref._person_form_events_service = MockPersonFormEvents()

        import redi.batch
        batch = reload(redi.batch)
        batch.check_input_file = lambda *args: None

        with self.assertRaises(DataFetched):
            redi_ref._run(config_file=None, configuration_directory='',
                      do_keep_gen_files=None, dry_run=True, get_emr_data=False,
                      settings=MockSettings(), data_folder=None,
                      database_path=None, resume=True, redcap_client=None,
                      report_courier=None, report_creator=None)


class MockSettings(object):
    def __getattr__(self, item):
        return '' if ('file' in item) else None

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import tempfile
import os
import logging
from lxml import etree

from redi import redi

class TestSortElementTree(unittest.TestCase):

    def setUp(self):
        redi.configure_logging('.')
        #redi.logger.setLevel(logging.DEBUG)

        # un-sorted XML file
        self.unsorted = """<?xml version="1.0" encoding="UTF-8"?>
<study>
   <subject>
        <NAME>PLATELET COUNT</NAME>
        <ORD_VALUE></ORD_VALUE>
        <STUDY_ID>999-0262</STUDY_ID>
        <redcapFormName>cbc</redcapFormName>
        <loinc_code>component_A</loinc_code>
        <DATE_TIME_STAMP>2013-12-03 00:00:00</DATE_TIME_STAMP>
    </subject>
    <subject>
        <NAME>PLATELET COUNT</NAME>
        <ORD_VALUE>123</ORD_VALUE>
        <STUDY_ID>999-0262</STUDY_ID>
        <redcapFormName>cbc</redcapFormName>
        <loinc_code>component_A</loinc_code>
        <DATE_TIME_STAMP>2013-12-01 00:00:00</DATE_TIME_STAMP>
    </subject>
    <subject>
        <NAME>PLATELET COUNT</NAME>
        <ORD_VALUE> </ORD_VALUE>
        <STUDY_ID>999-0262</STUDY_ID>
        <redcapFormName>cbc</redcapFormName>
        <loinc_code>component_A</loinc_code>
        <DATE_TIME_STAMP>2013-12-02 00:00:00</DATE_TIME_STAMP>
    </subject>
</study>
        """

        # we expect the following sorted tree
        self.sorted_tree = """<?xml version="1.0" encoding="UTF-8"?>
<study>
    <subject>
        <NAME>PLATELET COUNT</NAME>
        <ORD_VALUE>123</ORD_VALUE>
        <STUDY_ID>999-0262</STUDY_ID>
        <redcapFormName>cbc</redcapFormName>
        <loinc_code>component_A</loinc_code>
        <DATE_TIME_STAMP>2013-12-01 00:00:00</DATE_TIME_STAMP>
    </subject>
    <subject>
        <NAME>PLATELET COUNT</NAME>
        <ORD_VALUE> </ORD_VALUE>
        <STUDY_ID>999-0262</STUDY_ID>
        <redcapFormName>cbc</redcapFormName>
        <loinc_code>component_A</loinc_code>
        <DATE_TIME_STAMP>2013-12-02 00:00:00</DATE_TIME_STAMP>
    </subject>
    <subject>
        <NAME>PLATELET COUNT</NAME>
        <ORD_VALUE></ORD_VALUE>
        <STUDY_ID>999-0262</STUDY_ID>
        <redcapFormName>cbc</redcapFormName>
        <loinc_code>component_A</loinc_code>
        <DATE_TIME_STAMP>2013-12-03 00:00:00</DATE_TIME_STAMP>
    </subject>
</study>
        """
        self.dirpath = tempfile.mkdtemp()


    def test_sort_elementtree(self):
        tree_to_sort = etree.ElementTree(etree.fromstring(self.unsorted))
        redi.sort_element_tree(tree_to_sort, self.dirpath)

        par = etree.XMLParser(remove_blank_text = True)
        clean_expect = etree.XML(self.sorted_tree, parser=par)
        clean_result = etree.XML(etree.tostring(tree_to_sort), parser=par)
        self.assertEqual(etree.tostring(clean_expect), etree.tostring(clean_result))

    def tearDown(self):
        try:
            os.unlink(os.path.join(self.dirpath,
                "rawDataSortedBeforeCompression.xml"))
        except:
            print("setUp failed to unlink "\
                "file \'rawDataSortedBeforeCompression\'.xml")
        try:
            os.rmdir(self.dirpath)
        except OSError:
            raise LogException("Folder \'{}\' is not empty, hence cannot "\
                "be deleted.".format(self.dirpath))

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'


class TestUpdateFormCompletedFieldName(unittest.TestCase):

    def setUp(self):
        self.raw_xml = """<?xml version='1.0' encoding='US-ASCII'?>
<study>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>11/20/17</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMATOCRIT</Component_Name>
        <loinc_code>1534436</loinc_code>
        <Reference_Unit>%</Reference_Unit>
        <Result_Value>34.5</Result_Value>
    <timestamp/><redcapFormName>undefined</redcapFormName><eventName/><formDateField/><formCompletedFieldName/></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>11/20/17</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMOGLOBIN</Component_Name>
        <loinc_code>1534435</loinc_code>
        <Reference_Unit>g/dL</Reference_Unit>
        <Result_Value>11.3</Result_Value>
    <timestamp/><redcapFormName>cbc</redcapFormName><eventName/><formDateField/><formCompletedFieldName/></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>10/25/20</Collection_Date>
        <Collection_Time>12:38</Collection_Time>
        <Component_Name>BILIRUBIN DIRECT</Component_Name>
        <loinc_code>1558221</loinc_code>
        <Reference_Unit>mg/dL</Reference_Unit>
        <Result_Value>0.8</Result_Value>
    <timestamp/><redcapFormName>chemistry</redcapFormName><eventName/><formDateField/><formCompletedFieldName/></subject>
</study>
"""
        self.data_tree = etree.ElementTree(etree.fromstring(self.raw_xml))

        self.form_events = """<?xml version='1.0' encoding='US-ASCII'?>
<redcapProject>
	<name>My Test Project</name>
	<form>
		<name>cbc</name>
		<formDateField>cbc_lbdtc</formDateField>
		<formCompletedFieldName>cbc_complete</formCompletedFieldName>
		<event>
			<name>1_arm_1</name>
		</event>
		<event>
			<name>2_arm_1</name>
		</event>
		<event>
			<name>3_arm_1</name>
		</event>
    </form>
	<form>
		<name>chemistry</name>
		<formDateField>chemistry_lbdtc</formDateField>
		<formCompletedFieldName>chemistry_complete</formCompletedFieldName>
		<event>
			<name>1_arm_1</name>
		</event>
		<event>
			<name>2_arm_1</name>
		</event>
		<event>
			<name>3_arm_1</name>
		</event>
    </form>
</redcapProject>
"""


        self.form_events_tree = etree.ElementTree(etree.fromstring(self.form_events))

        self.output = """<?xml version='1.0' encoding='US-ASCII'?>
<study>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>11/20/17</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMATOCRIT</Component_Name>
        <loinc_code>1534436</loinc_code>
        <Reference_Unit>%</Reference_Unit>
        <Result_Value>34.5</Result_Value>
    <timestamp/><redcapFormName>undefined</redcapFormName><eventName/><formDateField/><formCompletedFieldName>undefined</formCompletedFieldName></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>11/20/17</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMOGLOBIN</Component_Name>
        <loinc_code>1534435</loinc_code>
        <Reference_Unit>g/dL</Reference_Unit>
        <Result_Value>11.3</Result_Value>
    <timestamp/><redcapFormName>cbc</redcapFormName><eventName/><formDateField/><formCompletedFieldName>cbc_complete</formCompletedFieldName></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>10/25/20</Collection_Date>
        <Collection_Time>12:38</Collection_Time>
        <Component_Name>BILIRUBIN DIRECT</Component_Name>
        <loinc_code>1558221</loinc_code>
        <Reference_Unit>mg/dL</Reference_Unit>
        <Result_Value>0.8</Result_Value>
    <timestamp/><redcapFormName>chemistry</redcapFormName><eventName/><formDateField/><formCompletedFieldName>chemistry_complete</formCompletedFieldName></subject>
</study>
"""
        self.expect = etree.tostring(etree.fromstring(self.output))
        return()


    def test_update_form_completed_field_name(self):
        redi.update_formcompletedfieldname(self.data_tree, self.form_events_tree, 'undefined')
        self.result = etree.tostring(self.data_tree)
        self.assertEqual(self.expect, self.result)

    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
@author : Mohan
email : mohan88@ufl.edu

This file tests for the function parse_raw_xml

'''
import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestValidateXmlFleAndExtractData(unittest.TestCase):

    def setUp(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        
        
    def test_validate_xml_file_and_extract_data_valid_xml(self):
        xml_file_name = "tempxmlFile.xml"
        temp_xml = open(xml_file_name, "wb")
        self.sampleXmlData = """<?xml version='1.0' encoding='US-ASCII'?>
<clinical_datum>
	<version>0.1.0</version>
	<Description>Test Description</Description>
	<components>
		<component>
			<description>Test Component 1</description>
			<source>
				<name>Comp_id</name>
				<value>123</value>
			</source>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>

		<component>
			<description>Test Component 2</description>
			<source>
				<name>Comp_id</name>
				<value>789</value>
			</source>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>
	</components>
</clinical_datum>"""
        temp_xml.write(self.sampleXmlData)
        temp_xml.close()
        
        xsd_file_name = "tempxsdFile.xsd"
        temp_xsd = open(xsd_file_name, "wb")
        self.sampleXsdData = """<xs:schema attributeFormDefault="unqualified" elementFormDefault="qualified" xmlns:xs="http://www.w3.org/2001/XMLSchema">
          <xs:element name="clinical_datum">
            <xs:complexType>
              <xs:sequence>
                <xs:element type="xs:string" name="version"/>
                <xs:element type="xs:string" name="Description"/>
                <xs:element name="components">
                  <xs:complexType>
                    <xs:sequence>
                      <xs:element name="component" maxOccurs="unbounded" minOccurs="0">
                        <xs:complexType>
                          <xs:sequence>
                            <xs:element type="xs:string" name="description"/>
                            <xs:element name="source">
                              <xs:complexType>
                                <xs:sequence>
                                  <xs:element type="xs:string" name="name"/>
                                  <xs:element type="xs:string" name="value"/>
                                </xs:sequence>
                              </xs:complexType>
                            </xs:element>
                            <xs:element name="target">
                              <xs:complexType>
                                <xs:sequence>
                                  <xs:element type="xs:string" name="name"/>
                                  <xs:element type="xs:string" name="value"/>
                                </xs:sequence>
                              </xs:complexType>
                            </xs:element>
                          </xs:sequence>
                        </xs:complexType>
                      </xs:element>
                    </xs:sequence>
                  </xs:complexType>
                </xs:element>
              </xs:sequence>
            </xs:complexType>
          </xs:element>
        </xs:schema>
        """
        temp_xsd.write(self.sampleXsdData)
        temp_xsd.close()
        self.result = redi.validate_xml_file_and_extract_data(xml_file_name, xsd_file_name)
        self.expect =  etree.tostring(self.result, method='html', pretty_print=True)
        self.output = """<clinical_datum>
	<version>0.1.0</version>
	<Description>Test Description</Description>
	<components>
		<component>
			<description>Test Component 1</description>
			<source>
				<name>Comp_id</name>
				<value>123</value>
			</source>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>

		<component>
			<description>Test Component 2</description>
			<source>
				<name>Comp_id</name>
				<value>789</value>
			</source>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>
	</components>
</clinical_datum>
"""
        self.assertEqual(self.expect, self.output)
        
        os.remove(xml_file_name)
        os.remove(xsd_file_name)
    
    def test_validate_xml_file_and_extract_data_invalid_xml(self):
        xml_file_name = "tempxmlFile.xml"
        temp_xml = open(xml_file_name, "wb")
        self.sampleXmlData = """<?xml version='1.0' encoding='US-ASCII'?>
<clinical_datum>
	<version>0.1.0</version>
	<Description>Test Description</Description>
	<components>
		<component>
			<description>Test Component 1</description>
			<source>
				<name>Comp_id</name>
				<value>123</value>
			</source>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>

		<component>
			<description>Test Component 2</description>
			<source>
				<name>Comp_id</name>
				<value>789</value>
			</source>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>
	</components>
</clinical_datum>"""
        temp_xml.write(self.sampleXmlData)
        temp_xml.close()
    
        xsd_file_name = "tempxsdFile.xsd"
        temp_xsd = open(xsd_file_name, "wb")
        self.sampleXsdData = """<xs:schema attributeFormDefault="unqualified" elementFormDefault="qualified" xmlns:xs="http://www.w3.org/2001/XMLSchema">
          <xs:element name="clinical_datum">
            <xs:complexType>
              <xs:sequence>
                <xs:element type="xs:string" name="version"/>
                <xs:element type="xs:string" name="Description"/>
                <xs:element name="components">
                  <xs:complexType>
                    <xs:sequence>
                      <xs:element name="component" maxOccurs="unbounded" minOccurs="0">
                        <xs:complexType>
                          <xs:sequence>
                            <xs:element type="xs:integer" name="description"/>
                            <xs:element name="source">
                              <xs:complexType>
                                <xs:sequence>
                                  <xs:element type="xs:string" name="name"/>
                                  <xs:element type="xs:string" name="value"/>
                                </xs:sequence>
                              </xs:complexType>
                            </xs:element>
                            <xs:element name="target">
                              <xs:complexType>
                                <xs:sequence>
                                  <xs:element type="xs:string" name="name"/>
                                  <xs:element type="xs:string" name="value"/>
                                </xs:sequence>
                              </xs:complexType>
                            </xs:element>
                          </xs:sequence>
                        </xs:complexType>
                      </xs:element>
                    </xs:sequence>
                  </xs:complexType>
                </xs:element>
              </xs:sequence>
            </xs:complexType>
          </xs:element>
        </xs:schema>
        """
        temp_xsd.write(self.sampleXsdData)
        temp_xsd.close()
        self.assertRaises(Exception,redi.validate_xml_file_and_extract_data,xml_file_name, xsd_file_name)
        os.remove(xml_file_name)
        os.remove(xsd_file_name)
        
    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

"""
TestGenerateOutput.py:

   Verifies the correct functionality 
   of the `generate_output` function
"""

import unittest
import os
from lxml import etree
from redi import redi
from redi import upload
from redi.utils.redcapClient import RedcapClient


DEFAULT_DATA_DIRECTORY = os.getcwd()


class TestGenerateOutput(unittest.TestCase):

    def setUp(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)

    class dummyClass:
        def_field = 'test'

    def test_person_form_event(self):
        string_1_xml = """
<person_form_event>
    <person lab_id="999-0100">
        <study_id>100</study_id>
        <all_form_events>
            <form>
                <name>cbc</name>
                <event>
                    <name>1_arm_1</name>
                    <field>
                        <name>cbc_lbdtc</name>
                        <value>1905-10-01</value>
                    </field>
                </event>
            </form>
            <form>
                <name>inr</name>
                <event>
                    <name>1_arm_1</name>
                    <field>
                        <name>cbc_lbdtc</name>
                        <value>1905-10-01</value>
                    </field>
                </event>
            </form>
 
        </all_form_events>
    </person>
    <person lab_id="999-0099">
        <study_id>99</study_id>
        <all_form_events>
            <form>
                <name>cbc</name>
                <event>
                    <name>1_arm_1</name>
                    <field>
                        <name>cbc_lbdtc</name>
                        <value>1905-10-01</value>
                    </field>
                    <field>
                        <name>wbc_lborres</name>
                        <value>3.0</value>
                    </field>
                    <field>
                        <name>wbc_lbstat</name>
                        <value/>
                    </field>
                    <field>
                        <name>neut_lborres</name>
                        <value>500</value>
                    </field>
                    <field>
                        <name>neut_lbstat</name>
                        <value/>
                    </field>
                </event>
            </form>
            <form>
                <name>inr</name>
                <event>
                    <name>1_arm_1</name>
                    <field>
                        <name>inr_lbdtc</name>
                        <value>1906-12-01</value>
                    </field>
                    <field>
                        <name>inr_lborres</name>
                        <value/>
                    </field>
                    <field>
                        <name>inr_lbstat</name>
                        <value>NOT_DONE</value>
                    </field>
                </event>
            </form>
        </all_form_events>
    </person>
    <person lab_id="999-0098">
        <study_id>98</study_id>
        <all_form_events>
            <form>
                <name>cbc</name>
                <event>
                    <name>1_arm_1</name>
                    <field>
                        <name>cbc_lbdtc</name>
                        <value></value>
                    </field>
                </event>
            </form>
            <form>
                <name>inr</name>
                <event>
                    <name>1_arm_1</name>
                    <field>
                        <name>cbc_lbdtc</name>
                        <value>1905-10-01</value>
                    </field>
                </event>
            </form>
        </all_form_events>
    </person>

</person_form_event>

"""

        form_details    = {'Total_cbc_Forms': 2, 'Total_inr_Forms': 3}
        subject_details = {
                '98'  : {'Total_cbc_Forms' : 0, 'Total_inr_Forms' : 1, 'lab_id': "999-0098" },
                '99'  : {'Total_cbc_Forms' : 1, 'Total_inr_Forms' : 1, "lab_id": "999-0099" },
                '100' : {'Total_cbc_Forms' : 1, 'Total_inr_Forms' : 1, "lab_id": "999-0100" }
        } 
 
        report_data = {
            'total_subjects'        : 3,
            'form_details'          : form_details,
            'subject_details'       : subject_details,
            'errors'                : []
        }

        class MockSentEventIndex(object):
            def __init__(self):
                self.sent_events = []

            def __len__(self):
                return len(self.sent_events)

            def mark_sent(self, study_id_key, form_name, event_name):
                form_event_key = study_id_key, form_name, event_name
                self.sent_events.append(form_event_key)

            def was_sent(self, study_id_key, form_name, event_name):
                form_event_key = study_id_key, form_name, event_name
                return form_event_key in self.sent_events

        class MockRedcapClient(RedcapClient):
            def __init__(self):
                self.project = TestGenerateOutput.dummyClass()

            def get_data_from_redcap(self, records_to_fetch=None,
                                     events_to_fetch=None,
                                     fields_to_fetch=None,
                                     forms_to_fetch=None,
                                     return_format='xml'):
                raise NotImplementedError()

            def send_data_to_redcap(self, data, max_retry_count,
                overwrite=False):
                return """Data sent"""

        etree_1 = etree.ElementTree(etree.fromstring(string_1_xml))
        result = upload.generate_output(etree_1, MockRedcapClient(), 500,
                                        MockSentEventIndex(), 10)
        self.assertEqual(report_data['total_subjects'], result['total_subjects'])
        self.assertEqual(report_data['form_details'], result['form_details'])
        self.assertEqual(report_data['subject_details'], result['subject_details'])
        self.assertEqual(report_data['errors'], result['errors'])


if __name__ == "__main__":
    unittest.main()




#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import tempfile
import shutil
import os

from redi import redi
from redi.utils import SimpleConfigParser

class TestReadConfig(unittest.TestCase):

    def setUp(self):
        _configure_redi_logger()

    def test_read_config(self):
        configuration_directory = tempfile.mkdtemp()
        try:
            files = ['TestTranslationTable.xml', 'TestFormEvents.xml',
                     'TestRaw.xml', 'TestResearchIdToRedcapId.xml',
                     'TestComponentToLoincCode.xml']

            for path in files:
                open(os.path.join(configuration_directory, path), 'a').close()

            # SHALL NOT raise exceptions
            redi.read_config(None, configuration_directory, files)

        finally:
            shutil.rmtree(configuration_directory)

    def test_read_config_missing_file_exits(self):
        kwargs = {
            "config_file": None,
            "configuration_directory": "/path/to/missing/folder",
            "file_list": ["doesn't matter, not a file"]
        }
        self.assertRaises(SystemExit, redi.read_config, **kwargs)


    def test_to_bool(self):
        """
        Verify we can convert properly the boolean strings
        """
        self.assertRaises(ValueError, SimpleConfigParser.to_bool, (None))
        self.assertRaises(ValueError, SimpleConfigParser.to_bool, (True))
        self.assertRaises(ValueError, SimpleConfigParser.to_bool, (False))
        self.assertTrue(SimpleConfigParser.to_bool('true'))
        self.assertTrue(SimpleConfigParser.to_bool('t'))
        self.assertTrue(SimpleConfigParser.to_bool('1'))
        self.assertTrue(SimpleConfigParser.to_bool('y'))

        self.assertFalse(SimpleConfigParser.to_bool('false'))
        self.assertFalse(SimpleConfigParser.to_bool('f'))
        self.assertFalse(SimpleConfigParser.to_bool('0'))
        self.assertFalse(SimpleConfigParser.to_bool('n'))

    def test_settings(self):
        settings = SimpleConfigParser.SimpleConfigParser()

        # write settings to a file and then read them right back
        with tempfile.NamedTemporaryFile() as t:
            t.write("""translation_table_file = TestTranslationTable.xml
form_events_file = TestFormEvents.xml
raw_xml_file = TestRaw.xml
research_id_to_redcap_id = TestResearchIdToRedcapId.xml
component_to_loinc_code_xml = TestComponentToLoincCode.xml
redcap_uri = https://example.org/redcap/api/
token = ABCDEF878D219CFA5D3ADF7F9AB12345
smtp_host_for_outbound_mail = smtp.example.org
redcap_support_receiver_email = jdoe@example.com
smtp_port_for_outbound_mail = 22
emr_sftp_server_hostname = TESTSERVER
emr_sftp_server_username = user
emr_sftp_server_password = pswd
emr_sftp_project_name = sample
emr_data_file = data.csv
include_rule_errors_in_report = False
verify_ssl = False
""")
            t.seek(0)
            settings.read(t.name)

        settings.set_attributes()

        self.assertEqual(settings.translation_table_file,
                         "TestTranslationTable.xml")
        self.assertEqual(settings.form_events_file,
                         "TestFormEvents.xml")
        self.assertEqual(settings.raw_xml_file,
                         "TestRaw.xml")
        self.assertEqual(settings.redcap_uri,
                         "https://example.org/redcap/api/")
        self.assertEqual(settings.token,
                         "ABCDEF878D219CFA5D3ADF7F9AB12345")
        self.assertEqual(settings.smtp_host_for_outbound_mail,
                         "smtp.example.org")

        self.assertFalse(settings.include_rule_errors_in_report)
        self.assertFalse(settings.verify_ssl)

def _configure_redi_logger():
    class MockLogger(object):
        def error(self, *args, **kwargs):
            pass

    redi.logger = MockLogger()

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestCreateEmptyEventsForOneSubject(unittest.TestCase):

    def setUp(self):
        self.xml1_test1 = """<?xml version='1.0' encoding='US-ASCII'?>
        <redcapProject>
        <name>Project</name>
        <form>
		<name>cbc</name>
		<formDateField>cbc_lbdtc</formDateField>
		<formCompletedFieldName>cbc_complete</formCompletedFieldName>
		<formImportedFieldName>cbc_nximport</formImportedFieldName>
		<event>
    		<name>1_arm_1</name>
		</event>
		
		
        </form>
        <form>
            <name>chemistry</name>
            <formDateField>chem_lbdtc</formDateField>
            <formCompletedFieldName>chemistry_complete</formCompletedFieldName>
            <formImportedFieldName>chem_nximport</formImportedFieldName>
            <event>
		        <name>1_arm_1</name>
		    </event>
		
		
        </form>

	    <form>
		    <name>inr</name>
		    <formDateField>inr_lbdtc</formDateField>
		    <formCompletedFieldName>inr_complete</formCompletedFieldName>
		    <formImportedFieldName>inr_nximport</formImportedFieldName>
		    <event>
		    <name>1_arm_1</name>
		    </event>
		
	       </form>
	
        </redcapProject>
        """
        self.data1_test1 = etree.ElementTree(etree.fromstring(self.xml1_test1))
        
        self.xml2_test1 = """<?xml version='1.0' encoding='US-ASCII'?>
        <rediFieldMap>
            <clinicalComponent>
                <clinicalComponentId>918</clinicalComponentId>
                <clinicalComponentName>LYMPHOCYTES</clinicalComponentName>
                <redcapFormName>cbc</redcapFormName>
                <redcapFieldNameValue>lymce_lborres</redcapFieldNameValue>
                <redcapFieldNameValueDescriptiveText>Lymphocytes (%)</redcapFieldNameValueDescriptiveText>
                <redcapFieldNameUnits>lymce_lborresu</redcapFieldNameUnits>
                <redcapFieldNameUnitsDescriptiveText>Lymphocytes (%) units</redcapFieldNameUnitsDescriptiveText>
                <redcapStatusFieldName>lymce_lbstat</redcapStatusFieldName>
                <lbtest>lymce_lbtest</lbtest>
                <lbtestcd>lymce_lbtestcd</lbtestcd>
            </clinicalComponent>
            <clinicalComponent>
                <clinicalComponentId>1534435</clinicalComponentId>
                <clinicalComponentName>HEMOGLOBIN</clinicalComponentName>
                <redcapFormName>cbc</redcapFormName>
                <redcapFieldNameValue>hemo_lborres</redcapFieldNameValue>
                <redcapFieldNameValueDescriptiveText>Hemoglobin</redcapFieldNameValueDescriptiveText>
                <redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits>
                <redcapFieldNameUnitsDescriptiveText>Hemoglobin units</redcapFieldNameUnitsDescriptiveText>
                <redcapStatusFieldName>hemo_lbstat</redcapStatusFieldName>
                <lbtest>hemo_lbtest</lbtest>
                <lbtestcd>hemo_lbtestcd</lbtestcd>
            </clinicalComponent>
            <clinicalComponent>
                <clinicalComponentId>971</clinicalComponentId>
                <clinicalComponentName>POTASSIUM</clinicalComponentName>
                <redcapFormName>chemistry</redcapFormName>
                <redcapFieldNameValue>k_lborres</redcapFieldNameValue>
                <redcapFieldNameValueDescriptiveText>Potassium</redcapFieldNameValueDescriptiveText>
                <redcapFieldNameUnits>k_lborresu</redcapFieldNameUnits>
                <redcapFieldNameUnitsDescriptiveText>Potassium units</redcapFieldNameUnitsDescriptiveText>
                <redcapStatusFieldName>k_lbstat</redcapStatusFieldName>
                <lbtest></lbtest>
                <lbtestcd></lbtestcd>
            </clinicalComponent>

            <clinicalComponent>
                <clinicalComponentId>1534098</clinicalComponentId>
                <clinicalComponentName>SODIUM</clinicalComponentName>
                <redcapFormName>chemistry</redcapFormName>
                <redcapFieldNameValue>sodium_lborres</redcapFieldNameValue>
                <redcapFieldNameValueDescriptiveText>Sodium</redcapFieldNameValueDescriptiveText>
                <redcapFieldNameUnits>sodium_lborresu</redcapFieldNameUnits>
                <redcapFieldNameUnitsDescriptiveText>Sodium units</redcapFieldNameUnitsDescriptiveText>
                <redcapStatusFieldName>sodium_lbstat</redcapStatusFieldName>
                <lbtest>Sodium</lbtest>
                <lbtestcd>SODIUM</lbtestcd>
            </clinicalComponent>
        </rediFieldMap>
        """
        self.data2_test1 = etree.ElementTree(etree.fromstring(self.xml2_test1))
        self.output_test1 = """<all_form_events><form>
        <name>cbc</name>
        <event>
            <name>1_arm_1</name>
        <status>unsent</status><field><name>lymce_lborres</name><value/></field><field><name>lymce_lborresu</name><value/></field><field><name>hemo_lborresu</name><value/></field><field><name>cbc_lbdtc</name><value/></field><field><name>cbc_nximport</name><value/></field><field><name>lymce_lbstat</name><value/></field><field><name>cbc_complete</name><value/></field><field><name>hemo_lbstat</name><value/></field><field><name>hemo_lborres</name><value/></field></event>
        
        
        </form>
        <form>
            <name>chemistry</name>
            <event>
                <name>1_arm_1</name>
            <status>unsent</status><field><name>k_lborres</name><value/></field><field><name>chem_lbdtc</name><value/></field><field><name>sodium_lborresu</name><value/></field><field><name>k_lbstat</name><value/></field><field><name>sodium_lbstat</name><value/></field><field><name>chem_nximport</name><value/></field><field><name>chemistry_complete</name><value/></field><field><name>k_lborresu</name><value/></field><field><name>sodium_lborres</name><value/></field></event>
        
        
        </form>

        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <status>unsent</status><field><name>inr_lbdtc</name><value/></field><field><name>inr_complete</name><value/></field><field><name>inr_nximport</name><value/></field></event>
        
           </form>
    
        </all_form_events>"""
        self.expect_test1 = etree.tostring(etree.fromstring(self.output_test1))
        
        self.xml1_test2 = """<?xml version='1.0' encoding='US-ASCII'?>
        <redcapProject>
        <name>Project</name>
        <form>
		<name>cbc</name>
		<event>
    		<name>1_arm_1</name>
		</event>
		
		
        </form>
        <form>
            <name>chemistry</name>
            <event>
		        <name>1_arm_1</name>
		    </event>
		
		
        </form>

	    <form>
		    <name>inr</name>
		    <event>
		    <name>1_arm_1</name>
		    </event>
		
	       </form>
	
        </redcapProject>
        """
        self.data1_test2 = etree.ElementTree(etree.fromstring(self.xml1_test2))
        
        self.xml2_test2 = """<?xml version='1.0' encoding='US-ASCII'?>
        <rediFieldMap>
            <clinicalComponent>
                <clinicalComponentId>918</clinicalComponentId>
                <clinicalComponentName>LYMPHOCYTES</clinicalComponentName>
                <redcapFormName>cbc</redcapFormName>
                <lbtest>lymce_lbtest</lbtest>
                <lbtestcd>lymce_lbtestcd</lbtestcd>
            </clinicalComponent>
            <clinicalComponent>
                <clinicalComponentId>1534435</clinicalComponentId>
                <clinicalComponentName>HEMOGLOBIN</clinicalComponentName>
                <redcapFormName>cbc</redcapFormName>
                <lbtest>hemo_lbtest</lbtest>
                <lbtestcd>hemo_lbtestcd</lbtestcd>
            </clinicalComponent>
            <clinicalComponent>
                <clinicalComponentId>971</clinicalComponentId>
                <clinicalComponentName>POTASSIUM</clinicalComponentName>
                <redcapFormName>chemistry</redcapFormName>
                <lbtest></lbtest>
                <lbtestcd></lbtestcd>
            </clinicalComponent>

            <clinicalComponent>
                <clinicalComponentId>1534098</clinicalComponentId>
                <clinicalComponentName>SODIUM</clinicalComponentName>
                <redcapFormName>chemistry</redcapFormName>
                <lbtest>Sodium</lbtest>
                <lbtestcd>SODIUM</lbtestcd>
            </clinicalComponent>
        </rediFieldMap>
        """
        self.data2_test2 = etree.ElementTree(etree.fromstring(self.xml2_test2))
        self.output_test2 = """<all_form_events><form>
        <name>cbc</name>
        <event>
        <name>1_arm_1</name>
        <field><name>lymce_lborres</name><value/></field><field><name>lymce_lborresu</name><value/></field><field><name>hemo_lborresu</name><value/></field><field><name>cbc_lbdtc</name><value/></field><field><name>cbc_nximport</name><value/></field><field><name>lymce_lbstat</name><value/></field><field><name>cbc_complete</name><value/></field><field><name>hemo_lbstat</name><value/></field><field><name>hemo_lborres</name><value/></field></event>


        </form>
        <form>
            <name>chemistry</name>
            <event>
                <name>1_arm_1</name>
                <field><name>k_lborres</name><value/></field><field><name>chem_lbdtc</name><value/></field><field><name>sodium_lborresu</name><value/></field><field><name>k_lbstat</name><value/></field><field><name>sodium_lbstat</name><value/></field><field><name>chem_nximport</name><value/></field><field><name>chemistry_complete</name><value/></field><field><name>k_lborresu</name><value/></field><field><name>sodium_lborres</name><value/></field></event>


        </form>

        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field><field><name>inr_complete</name><value/></field><field><name>inr_nximport</name><value/></field></event>
        
           </form>
    
        </all_form_events>
        """
        self.expect_test2 = etree.tostring(etree.fromstring(self.output_test2))
        return()


    def test_create_empty_events_for_one_subject_all_fields_input(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.result = etree.tostring(redi.create_empty_events_for_one_subject(self.data1_test1,self.data2_test1))
        self.result = ''.join(self.result.split())
        self.expect_test1 = ''.join(self.expect_test1.split())
        self.assertEqual(self.expect_test1, self.result)

    def test_create_empty_events_for_one_subject_no_fields_input(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.assertRaises(Exception,redi.create_empty_events_for_one_subject,self.data1_test2,self.data2_test2)

    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
@author : Mohan
email : mohan88@ufl.edu

This file tests for the function parse_raw_xml

'''
import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestParseRawXml(unittest.TestCase):

    def setUp(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.sampleData = """<study>
        <subject>
            <NAME>TestSubject1</NAME>
            <loinc_code>test1</loinc_code>
            <RESULT>0.12</RESULT>
            <REFERENCE_LOW>0.34</REFERENCE_LOW>
            <REFERENCE_HIGH>5.60</REFERENCE_HIGH>
            <REFERENCE_UNIT>mIU/L</REFERENCE_UNIT>
            <DATE_TIME_STAMP>7891-11-30 11:12:00</DATE_TIME_STAMP>
            <STUDY_ID>1234-5678</STUDY_ID>
        </subject>
        <subject>
            <NAME>TestSubject2</NAME>
            <loinc_code>test2</loinc_code>
            <RESULT>8.7</RESULT>
            <REFERENCE_LOW/>
            <REFERENCE_HIGH/>
            <REFERENCE_UNIT>ml</REFERENCE_UNIT>
            <DATE_TIME_STAMP>1909-8-27 16:13:00</DATE_TIME_STAMP>
            <STUDY_ID>987-654</STUDY_ID>
        </subject>
    </study>
"""
        
    def test_parse_raw_xml_with_file(self):
        file_name = "tempFile.xml"
        temp_xml = open(file_name, "wb")
        temp_xml.write(self.sampleData)
        temp_xml.close()
        result = redi.parse_raw_xml('tempFile.xml')
        result_str = etree.tostring(result, method='xml', pretty_print=True)
        self.assertEqual(self.sampleData, result_str)
        os.remove(file_name)
    
    def test_parse_raw_xml_without_file(self):
        self.assertRaises(Exception,redi.parse_raw_xml,'')  
        
    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import logging
import time
import sys

from redcap import Project, RedcapError
from requests import RequestException
from requests.packages.urllib3.exceptions import MaxRetryError

# Configure module's logger
logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class RedcapClient(object):
    """ Client for a REDCap server.

    :param redcap_uri: URI for to REDCap server's API
    :param token: API Token for a REDCap project.
    :param verify_ssl: verify the SSL certificate? (default: True)
    :raises RedcapError: if we failed to get the project's metadata
    :raises RequestException: if some other network-related failure occurs
    """
    def __init__(self, redcap_uri, token, verify_ssl=True):
        self.redcap_uri = redcap_uri
        msg = 'Initializing redcap interface for: ' + redcap_uri
        logger.info(msg)
        self.token = token
        self.verify_ssl = verify_ssl

        try:
            self.project = Project(redcap_uri, token, "", verify_ssl)
            logger.info("redcap interface initialzed")
        except (RequestException, RedcapError) as e:
            logger.exception(e.message)
            raise

    def get_data_from_redcap(
            self,
            records_to_fetch=None,
            events_to_fetch=None,
            fields_to_fetch=None,
            forms_to_fetch=None,
            return_format='xml'):
        """ Exports REDCap records.

        :param records_to_fetch: if specified, only includes records in this
            list. Otherwise, includes all records.
        :type records_to_fetch: list or None

        :param events_to_fetch: if specified, only includes events in this list.
            Otherwise, includes all events.
        :type events_to_fetch: list or None

        :param fields_to_fetch: if specified, only includes fields in this list.
            Otherwise, includes all fields
        :type fields_to_fetch: list or None

        :param forms_to_fetch: if specified, only includes forms in this list.
            Otherwise, includes all forms.
        :type forms_to_fetch: list or None

        :param return_format: specifies the format of the REDCap response
            (default: xml)

        :return: response
        """
        logger.info('getting data from redcap')
        try:
            response = self.project.export_records(
                records=records_to_fetch,
                events=events_to_fetch,
                fields=fields_to_fetch,
                forms=forms_to_fetch,
                format=return_format)
        except RedcapError as e:
            logger.debug(e.message)
        return response

    def send_data_to_redcap(self, data, max_retry_count, overwrite=False,
        retry_count=0):
        """ Sends records to REDCap.

        :param list of dict objects data: records to send.
        :param bool overwrite: treat blank values as intentional?
            (default: False) When sending a record, if a field is blank, by
            default REDCap will not overwrite any existing value with a blank.
        :return: response
        :raises RedcapError: if failed to send records for any reason.
        :If MaxRetryError is caught, the function will try resending the same
         data for a maximum of max_retry_count times before exitting. For each
         attempt the wait time before sending is (the_attempt_no * 6)
        """
        overwrite_value = 'overwrite' if overwrite else 'normal'

        try:
            # The following line simulates github issue #108:
            # raise MaxRetryError('', 'localhost:8998', None)
            response = self.project.import_records(data,
                overwrite=overwrite_value)
            return response
        except MaxRetryError as e:
            logger.debug(e.message + ", Attempt no.: " + str(retry_count))
            if (retry_count == max_retry_count):
                message = "Exiting since network connection timed out after"\
                " reaching the maximum retry limit for resending data."
                logger.debug(message)
                sys.exit(message)
            # wait for some time before resending data
            time.sleep(retry_count*6)
            self.send_data_to_redcap(data, max_retry_count, 'overwrite',
                retry_count+1)
        except RedcapError as e:
            logger.debug(e.message)
            raise
